# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil.resources import Resource, ResourceList


class PriorityClass(Resource):
    api_version: Annotated[
        Optional[Literal["scheduling.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "scheduling.k8s.io/v1"
    description: Annotated[
        Optional[str],
        Field(
            description="description is an arbitrary string that usually provides guidelines on when this priority class should be used."
        ),
    ] = None
    global_default: Annotated[
        Optional[bool],
        Field(
            alias="globalDefault",
            description="globalDefault specifies whether this PriorityClass should be considered as the default priority for pods that do not have any priority class. Only one PriorityClass can be marked as `globalDefault`. However, if more than one PriorityClasses exists with their `globalDefault` field set to true, the smallest value of such global default PriorityClasses will be used as the default priority.",
        ),
    ] = None
    kind: Annotated[
        Optional[Literal["PriorityClass"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "PriorityClass"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    preemption_policy: Annotated[
        Optional[str],
        Field(
            alias="preemptionPolicy",
            description="preemptionPolicy is the Policy for preempting pods with lower priority. One of Never, PreemptLowerPriority. Defaults to PreemptLowerPriority if unset.",
        ),
    ] = None
    value: Annotated[
        int,
        Field(
            description="value represents the integer value of this priority class. This is the actual priority that pods receive when they have the name of this class in their pod spec."
        ),
    ]


PriorityClassList = ResourceList["PriorityClass"]



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, Dict, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList

from ..core import v1


class Scheduling(BaseModel):
    node_selector: Annotated[
        Optional[Dict[str, str]],
        Field(
            alias="nodeSelector",
            description="nodeSelector lists labels that must be present on nodes that support this RuntimeClass. Pods using this RuntimeClass can only be scheduled to a node matched by this selector. The RuntimeClass nodeSelector is merged with a pod's existing nodeSelector. Any conflicts will cause the pod to be rejected in admission.",
        ),
    ] = None
    tolerations: Annotated[
        Optional[List[v1.Toleration]],
        Field(
            description="tolerations are appended (excluding duplicates) to pods running with this RuntimeClass during admission, effectively unioning the set of nodes tolerated by the pod and the RuntimeClass."
        ),
    ] = None


class Overhead(BaseModel):
    pod_fixed: Annotated[
        Optional[Dict[str, apimachinery.Quantity]],
        Field(
            alias="podFixed",
            description="podFixed represents the fixed resource overhead associated with running a pod.",
        ),
    ] = None


class RuntimeClass(Resource):
    api_version: Annotated[
        Optional[Literal["node.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "node.k8s.io/v1"
    handler: Annotated[
        str,
        Field(
            description='handler specifies the underlying runtime and configuration that the CRI implementation will use to handle pods of this class. The possible values are specific to the node & CRI configuration.  It is assumed that all handlers are available on every node, and handlers of the same name are equivalent on every node. For example, a handler called "runc" might specify that the runc OCI runtime (using native Linux containers) will be used to run the containers in a pod. The Handler must be lowercase, conform to the DNS Label (RFC 1123) requirements, and is immutable.'
        ),
    ]
    kind: Annotated[
        Optional[Literal["RuntimeClass"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "RuntimeClass"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    overhead: Annotated[
        Optional[Overhead],
        Field(
            description="overhead represents the resource overhead associated with running a pod for a given RuntimeClass. For more details, see\n https://kubernetes.io/docs/concepts/scheduling-eviction/pod-overhead/"
        ),
    ] = None
    scheduling: Annotated[
        Optional[Scheduling],
        Field(
            description="scheduling holds the scheduling constraints to ensure that pods running with this RuntimeClass are scheduled to nodes that support it. If scheduling is nil, this RuntimeClass is assumed to be supported by all nodes."
        ),
    ] = None


RuntimeClassList = ResourceList["RuntimeClass"]



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList

from ..core import v1


class EventSeries(BaseModel):
    count: Annotated[
        int,
        Field(
            description="count is the number of occurrences in this series up to the last heartbeat time."
        ),
    ]
    last_observed_time: Annotated[
        apimachinery.MicroTime,
        Field(
            alias="lastObservedTime",
            description="lastObservedTime is the time when last Event from the series was seen before last heartbeat.",
        ),
    ]


class Event(Resource):
    action: Annotated[
        Optional[str],
        Field(
            description="action is what action was taken/failed regarding to the regarding object. It is machine-readable. This field cannot be empty for new Events and it can have at most 128 characters."
        ),
    ] = None
    api_version: Annotated[
        Optional[Literal["events.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "events.k8s.io/v1"
    deprecated_count: Annotated[
        Optional[int],
        Field(
            alias="deprecatedCount",
            description="deprecatedCount is the deprecated field assuring backward compatibility with core.v1 Event type.",
        ),
    ] = None
    deprecated_first_timestamp: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="deprecatedFirstTimestamp",
            description="deprecatedFirstTimestamp is the deprecated field assuring backward compatibility with core.v1 Event type.",
        ),
    ] = None
    deprecated_last_timestamp: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="deprecatedLastTimestamp",
            description="deprecatedLastTimestamp is the deprecated field assuring backward compatibility with core.v1 Event type.",
        ),
    ] = None
    deprecated_source: Annotated[
        Optional[v1.EventSource],
        Field(
            alias="deprecatedSource",
            description="deprecatedSource is the deprecated field assuring backward compatibility with core.v1 Event type.",
        ),
    ] = None
    event_time: Annotated[
        apimachinery.MicroTime,
        Field(
            alias="eventTime",
            description="eventTime is the time when this Event was first observed. It is required.",
        ),
    ]
    kind: Annotated[
        Optional[Literal["Event"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "Event"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    note: Annotated[
        Optional[str],
        Field(
            description="note is a human-readable description of the status of this operation. Maximal length of the note is 1kB, but libraries should be prepared to handle values up to 64kB."
        ),
    ] = None
    reason: Annotated[
        Optional[str],
        Field(
            description="reason is why the action was taken. It is human-readable. This field cannot be empty for new Events and it can have at most 128 characters."
        ),
    ] = None
    regarding: Annotated[
        Optional[v1.ObjectReference],
        Field(
            description="regarding contains the object this Event is about. In most cases it's an Object reporting controller implements, e.g. ReplicaSetController implements ReplicaSets and this event is emitted because it acts on some changes in a ReplicaSet object."
        ),
    ] = None
    related: Annotated[
        Optional[v1.ObjectReference],
        Field(
            description="related is the optional secondary object for more complex actions. E.g. when regarding object triggers a creation or deletion of related object."
        ),
    ] = None
    reporting_controller: Annotated[
        Optional[str],
        Field(
            alias="reportingController",
            description="reportingController is the name of the controller that emitted this Event, e.g. `kubernetes.io/kubelet`. This field cannot be empty for new Events.",
        ),
    ] = None
    reporting_instance: Annotated[
        Optional[str],
        Field(
            alias="reportingInstance",
            description="reportingInstance is the ID of the controller instance, e.g. `kubelet-xyzf`. This field cannot be empty for new Events and it can have at most 128 characters.",
        ),
    ] = None
    series: Annotated[
        Optional[EventSeries],
        Field(
            description="series is data about the Event series this event represents or nil if it's a singleton Event."
        ),
    ] = None
    type: Annotated[
        Optional[str],
        Field(
            description="type is the type of this event (Normal, Warning), new types could be added in the future. It is machine-readable. This field cannot be empty for new Events."
        ),
    ] = None


EventList = ResourceList["Event"]



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList


class CrossVersionObjectReference(BaseModel):
    api_version: Annotated[
        Optional[str],
        Field(
            alias="apiVersion",
            description="apiVersion is the API version of the referent",
        ),
    ] = None
    kind: Annotated[
        str,
        Field(
            description="kind is the kind of the referent; More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ]
    name: Annotated[
        str,
        Field(
            description="name is the name of the referent; More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names"
        ),
    ]


class HorizontalPodAutoscalerSpec(BaseModel):
    max_replicas: Annotated[
        int,
        Field(
            alias="maxReplicas",
            description="maxReplicas is the upper limit for the number of pods that can be set by the autoscaler; cannot be smaller than MinReplicas.",
        ),
    ]
    min_replicas: Annotated[
        Optional[int],
        Field(
            alias="minReplicas",
            description="minReplicas is the lower limit for the number of replicas to which the autoscaler can scale down.  It defaults to 1 pod.  minReplicas is allowed to be 0 if the alpha feature gate HPAScaleToZero is enabled and at least one Object or External metric is configured.  Scaling is active as long as at least one metric value is available.",
        ),
    ] = None
    scale_target_ref: Annotated[
        CrossVersionObjectReference,
        Field(
            alias="scaleTargetRef",
            description="reference to scaled resource; horizontal pod autoscaler will learn the current resource consumption and will set the desired number of pods by using its Scale subresource.",
        ),
    ]
    target_cpu_utilization_percentage: Annotated[
        Optional[int],
        Field(
            alias="targetCPUUtilizationPercentage",
            description="targetCPUUtilizationPercentage is the target average CPU utilization (represented as a percentage of requested CPU) over all the pods; if not specified the default autoscaling policy will be used.",
        ),
    ] = None


class ScaleSpec(BaseModel):
    replicas: Annotated[
        Optional[int],
        Field(description="replicas is the desired number of instances for the scaled object."),
    ] = None


class ScaleStatus(BaseModel):
    replicas: Annotated[
        int,
        Field(
            description="replicas is the actual number of observed instances of the scaled object."
        ),
    ]
    selector: Annotated[
        Optional[str],
        Field(
            description="selector is the label query over pods that should match the replicas count. This is same as the label selector but in the string format to avoid introspection by clients. The string will be in the same format as the query-param syntax. More info about label selectors: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/"
        ),
    ] = None


class HorizontalPodAutoscalerStatus(BaseModel):
    current_cpu_utilization_percentage: Annotated[
        Optional[int],
        Field(
            alias="currentCPUUtilizationPercentage",
            description="currentCPUUtilizationPercentage is the current average CPU utilization over all pods, represented as a percentage of requested CPU, e.g. 70 means that an average pod is using now 70% of its requested CPU.",
        ),
    ] = None
    current_replicas: Annotated[
        int,
        Field(
            alias="currentReplicas",
            description="currentReplicas is the current number of replicas of pods managed by this autoscaler.",
        ),
    ]
    desired_replicas: Annotated[
        int,
        Field(
            alias="desiredReplicas",
            description="desiredReplicas is the  desired number of replicas of pods managed by this autoscaler.",
        ),
    ]
    last_scale_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastScaleTime",
            description="lastScaleTime is the last time the HorizontalPodAutoscaler scaled the number of pods; used by the autoscaler to control how often the number of pods is changed.",
        ),
    ] = None
    observed_generation: Annotated[
        Optional[int],
        Field(
            alias="observedGeneration",
            description="observedGeneration is the most recent generation observed by this autoscaler.",
        ),
    ] = None


class HorizontalPodAutoscaler(Resource):
    api_version: Annotated[
        Optional[Literal["autoscaling/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "autoscaling/v1"
    kind: Annotated[
        Optional[Literal["HorizontalPodAutoscaler"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "HorizontalPodAutoscaler"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[HorizontalPodAutoscalerSpec],
        Field(
            description="spec defines the behaviour of autoscaler. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status."
        ),
    ] = None
    status: Annotated[
        Optional[HorizontalPodAutoscalerStatus],
        Field(description="status is the current information about the autoscaler."),
    ] = None


HorizontalPodAutoscalerList = ResourceList["HorizontalPodAutoscaler"]


class Scale(Resource):
    api_version: Annotated[
        Optional[Literal["autoscaling/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "autoscaling/v1"
    kind: Annotated[
        Optional[Literal["Scale"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "Scale"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object metadata; More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata."
        ),
    ] = None
    spec: Annotated[
        Optional[ScaleSpec],
        Field(
            description="spec defines the behavior of the scale. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status."
        ),
    ] = None
    status: Annotated[
        Optional[ScaleStatus],
        Field(
            description="status is the current status of the scale. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status. Read-only."
        ),
    ] = None



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList

from ..core import v1


class IPBlock(BaseModel):
    cidr: Annotated[
        str,
        Field(
            description='cidr is a string representing the IPBlock Valid examples are "192.168.1.0/24" or "2001:db8::/64"'
        ),
    ]
    except_: Annotated[
        Optional[List[str]],
        Field(
            alias="except",
            description='except is a slice of CIDRs that should not be included within an IPBlock Valid examples are "192.168.1.0/24" or "2001:db8::/64" Except values will be rejected if they are outside the cidr range',
        ),
    ] = None


class IngressClassParametersReference(BaseModel):
    api_group: Annotated[
        Optional[str],
        Field(
            alias="apiGroup",
            description="apiGroup is the group for the resource being referenced. If APIGroup is not specified, the specified Kind must be in the core API group. For any other third-party types, APIGroup is required.",
        ),
    ] = None
    kind: Annotated[str, Field(description="kind is the type of resource being referenced.")]
    name: Annotated[str, Field(description="name is the name of resource being referenced.")]
    namespace: Annotated[
        Optional[str],
        Field(
            description='namespace is the namespace of the resource being referenced. This field is required when scope is set to "Namespace" and must be unset when scope is set to "Cluster".'
        ),
    ] = None
    scope: Annotated[
        Optional[str],
        Field(
            description='scope represents if this refers to a cluster or namespace scoped resource. This may be set to "Cluster" (default) or "Namespace".'
        ),
    ] = None


class IngressClassSpec(BaseModel):
    controller: Annotated[
        Optional[str],
        Field(
            description='controller refers to the name of the controller that should handle this class. This allows for different "flavors" that are controlled by the same controller. For example, you may have different parameters for the same implementing controller. This should be specified as a domain-prefixed path no more than 250 characters in length, e.g. "acme.io/ingress-controller". This field is immutable.'
        ),
    ] = None
    parameters: Annotated[
        Optional[IngressClassParametersReference],
        Field(
            description="parameters is a link to a custom resource containing additional configuration for the controller. This is optional if the controller does not require extra parameters."
        ),
    ] = None


class IngressPortStatus(BaseModel):
    error: Annotated[
        Optional[str],
        Field(
            description="error is to record the problem with the service port The format of the error shall comply with the following rules: - built-in error values shall be specified in this file and those shall use\n  CamelCase names\n- cloud provider specific error values must have names that comply with the\n  format foo.example.com/CamelCase."
        ),
    ] = None
    port: Annotated[int, Field(description="port is the port number of the ingress port.")]
    protocol: Annotated[
        str,
        Field(
            description='protocol is the protocol of the ingress port. The supported values are: "TCP", "UDP", "SCTP"'
        ),
    ]


class IngressTLS(BaseModel):
    hosts: Annotated[
        Optional[List[str]],
        Field(
            description="hosts is a list of hosts included in the TLS certificate. The values in this list must match the name/s used in the tlsSecret. Defaults to the wildcard host setting for the loadbalancer controller fulfilling this Ingress, if left unspecified."
        ),
    ] = None
    secret_name: Annotated[
        Optional[str],
        Field(
            alias="secretName",
            description='secretName is the name of the secret used to terminate TLS traffic on port 443. Field is left optional to allow TLS routing based on SNI hostname alone. If the SNI host in a listener conflicts with the "Host" header field used by an IngressRule, the SNI host is used for termination and value of the "Host" header is used for routing.',
        ),
    ] = None


class ServiceBackendPort(BaseModel):
    name: Annotated[
        Optional[str],
        Field(
            description='name is the name of the port on the Service. This is a mutually exclusive setting with "Number".'
        ),
    ] = None
    number: Annotated[
        Optional[int],
        Field(
            description='number is the numerical port number (e.g. 80) on the Service. This is a mutually exclusive setting with "Name".'
        ),
    ] = None


class IngressLoadBalancerIngress(BaseModel):
    hostname: Annotated[
        Optional[str],
        Field(description="hostname is set for load-balancer ingress points that are DNS based."),
    ] = None
    ip: Annotated[
        Optional[str],
        Field(description="ip is set for load-balancer ingress points that are IP based."),
    ] = None
    ports: Annotated[
        Optional[List[IngressPortStatus]],
        Field(
            description="ports provides information about the ports exposed by this LoadBalancer."
        ),
    ] = None


class IngressLoadBalancerStatus(BaseModel):
    ingress: Annotated[
        Optional[List[IngressLoadBalancerIngress]],
        Field(description="ingress is a list containing ingress points for the load-balancer."),
    ] = None


class IngressServiceBackend(BaseModel):
    name: Annotated[
        str,
        Field(
            description="name is the referenced service. The service must exist in the same namespace as the Ingress object."
        ),
    ]
    port: Annotated[
        Optional[ServiceBackendPort],
        Field(
            description="port of the referenced service. A port name or port number is required for a IngressServiceBackend."
        ),
    ] = None


class IngressStatus(BaseModel):
    load_balancer: Annotated[
        Optional[IngressLoadBalancerStatus],
        Field(
            alias="loadBalancer",
            description="loadBalancer contains the current status of the load-balancer.",
        ),
    ] = None


class NetworkPolicyPort(BaseModel):
    end_port: Annotated[
        Optional[int],
        Field(
            alias="endPort",
            description="endPort indicates that the range of ports from port to endPort if set, inclusive, should be allowed by the policy. This field cannot be defined if the port field is not defined or if the port field is defined as a named (string) port. The endPort must be equal or greater than port.",
        ),
    ] = None
    port: Annotated[
        Optional[apimachinery.IntOrString],
        Field(
            description="port represents the port on the given protocol. This can either be a numerical or named port on a pod. If this field is not provided, this matches all port names and numbers. If present, only traffic on the specified protocol AND port will be matched."
        ),
    ] = None
    protocol: Annotated[
        Optional[str],
        Field(
            description="protocol represents the protocol (TCP, UDP, or SCTP) which traffic must match. If not specified, this field defaults to TCP."
        ),
    ] = None


class IngressBackend(BaseModel):
    resource: Annotated[
        Optional[v1.TypedLocalObjectReference],
        Field(
            description='resource is an ObjectRef to another Kubernetes resource in the namespace of the Ingress object. If resource is specified, a service.Name and service.Port must not be specified. This is a mutually exclusive setting with "Service".'
        ),
    ] = None
    service: Annotated[
        Optional[IngressServiceBackend],
        Field(
            description='service references a service as a backend. This is a mutually exclusive setting with "Resource".'
        ),
    ] = None


class IngressClass(Resource):
    api_version: Annotated[
        Optional[Literal["networking.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "networking.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["IngressClass"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "IngressClass"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[IngressClassSpec],
        Field(
            description="spec is the desired state of the IngressClass. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None


IngressClassList = ResourceList["IngressClass"]


class NetworkPolicyPeer(BaseModel):
    ip_block: Annotated[
        Optional[IPBlock],
        Field(
            alias="ipBlock",
            description="ipBlock defines policy on a particular IPBlock. If this field is set then neither of the other fields can be.",
        ),
    ] = None
    namespace_selector: Annotated[
        Optional[apimachinery.LabelSelector],
        Field(
            alias="namespaceSelector",
            description="namespaceSelector selects namespaces using cluster-scoped labels. This field follows standard label selector semantics; if present but empty, it selects all namespaces.\n\nIf podSelector is also set, then the NetworkPolicyPeer as a whole selects the pods matching podSelector in the namespaces selected by namespaceSelector. Otherwise it selects all pods in the namespaces selected by namespaceSelector.",
        ),
    ] = None
    pod_selector: Annotated[
        Optional[apimachinery.LabelSelector],
        Field(
            alias="podSelector",
            description="podSelector is a label selector which selects pods. This field follows standard label selector semantics; if present but empty, it selects all pods.\n\nIf namespaceSelector is also set, then the NetworkPolicyPeer as a whole selects the pods matching podSelector in the Namespaces selected by NamespaceSelector. Otherwise it selects the pods matching podSelector in the policy's own namespace.",
        ),
    ] = None


class HTTPIngressPath(BaseModel):
    backend: Annotated[
        IngressBackend,
        Field(
            description="backend defines the referenced service endpoint to which the traffic will be forwarded to."
        ),
    ]
    path: Annotated[
        Optional[str],
        Field(
            description='path is matched against the path of an incoming request. Currently it can contain characters disallowed from the conventional "path" part of a URL as defined by RFC 3986. Paths must begin with a \'/\' and must be present when using PathType with value "Exact" or "Prefix".'
        ),
    ] = None
    path_type: Annotated[
        str,
        Field(
            alias="pathType",
            description="pathType determines the interpretation of the path matching. PathType can be one of the following values: * Exact: Matches the URL path exactly. * Prefix: Matches based on a URL path prefix split by '/'. Matching is\n  done on a path element by element basis. A path element refers is the\n  list of labels in the path split by the '/' separator. A request is a\n  match for path p if every p is an element-wise prefix of p of the\n  request path. Note that if the last element of the path is a substring\n  of the last element in request path, it is not a match (e.g. /foo/bar\n  matches /foo/bar/baz, but does not match /foo/barbaz).\n* ImplementationSpecific: Interpretation of the Path matching is up to\n  the IngressClass. Implementations can treat this as a separate PathType\n  or treat it identically to Prefix or Exact path types.\nImplementations are required to support all path types.",
        ),
    ]


class HTTPIngressRuleValue(BaseModel):
    paths: Annotated[
        List[HTTPIngressPath],
        Field(description="paths is a collection of paths that map requests to backends."),
    ]


class IngressRule(BaseModel):
    host: Annotated[
        Optional[str],
        Field(
            description='host is the fully qualified domain name of a network host, as defined by RFC 3986. Note the following deviations from the "host" part of the URI as defined in RFC 3986: 1. IPs are not allowed. Currently an IngressRuleValue can only apply to\n   the IP in the Spec of the parent Ingress.\n2. The `:` delimiter is not respected because ports are not allowed.\n\t  Currently the port of an Ingress is implicitly :80 for http and\n\t  :443 for https.\nBoth these may change in the future. Incoming requests are matched against the host before the IngressRuleValue. If the host is unspecified, the Ingress routes all traffic based on the specified IngressRuleValue.\n\nhost can be "precise" which is a domain name without the terminating dot of a network host (e.g. "foo.bar.com") or "wildcard", which is a domain name prefixed with a single wildcard label (e.g. "*.foo.com"). The wildcard character \'*\' must appear by itself as the first DNS label and matches only a single label. You cannot have a wildcard label by itself (e.g. Host == "*"). Requests will be matched against the Host field in the following way: 1. If host is precise, the request matches this rule if the http host header is equal to Host. 2. If host is a wildcard, then the request matches this rule if the http host header is to equal to the suffix (removing the first label) of the wildcard rule.'
        ),
    ] = None
    http: Optional[HTTPIngressRuleValue] = None


class IngressSpec(BaseModel):
    default_backend: Annotated[
        Optional[IngressBackend],
        Field(
            alias="defaultBackend",
            description="defaultBackend is the backend that should handle requests that don't match any rule. If Rules are not specified, DefaultBackend must be specified. If DefaultBackend is not set, the handling of requests that do not match any of the rules will be up to the Ingress controller.",
        ),
    ] = None
    ingress_class_name: Annotated[
        Optional[str],
        Field(
            alias="ingressClassName",
            description="ingressClassName is the name of an IngressClass cluster resource. Ingress controller implementations use this field to know whether they should be serving this Ingress resource, by a transitive connection (controller -> IngressClass -> Ingress resource). Although the `kubernetes.io/ingress.class` annotation (simple constant name) was never formally defined, it was widely supported by Ingress controllers to create a direct binding between Ingress controller and Ingress resources. Newly created Ingress resources should prefer using the field. However, even though the annotation is officially deprecated, for backwards compatibility reasons, ingress controllers should still honor that annotation if present.",
        ),
    ] = None
    rules: Annotated[
        Optional[List[IngressRule]],
        Field(
            description="rules is a list of host rules used to configure the Ingress. If unspecified, or no rule matches, all traffic is sent to the default backend."
        ),
    ] = None
    tls: Annotated[
        Optional[List[IngressTLS]],
        Field(
            description="tls represents the TLS configuration. Currently the Ingress only supports a single TLS port, 443. If multiple members of this list specify different hosts, they will be multiplexed on the same port according to the hostname specified through the SNI TLS extension, if the ingress controller fulfilling the ingress supports SNI."
        ),
    ] = None


class NetworkPolicyEgressRule(BaseModel):
    ports: Annotated[
        Optional[List[NetworkPolicyPort]],
        Field(
            description="ports is a list of destination ports for outgoing traffic. Each item in this list is combined using a logical OR. If this field is empty or missing, this rule matches all ports (traffic not restricted by port). If this field is present and contains at least one item, then this rule allows traffic only if the traffic matches at least one port in the list."
        ),
    ] = None
    to: Annotated[
        Optional[List[NetworkPolicyPeer]],
        Field(
            description="to is a list of destinations for outgoing traffic of pods selected for this rule. Items in this list are combined using a logical OR operation. If this field is empty or missing, this rule matches all destinations (traffic not restricted by destination). If this field is present and contains at least one item, this rule allows traffic only if the traffic matches at least one item in the to list."
        ),
    ] = None


class NetworkPolicyIngressRule(BaseModel):
    from_: Annotated[
        Optional[List[NetworkPolicyPeer]],
        Field(
            alias="from",
            description="from is a list of sources which should be able to access the pods selected for this rule. Items in this list are combined using a logical OR operation. If this field is empty or missing, this rule matches all sources (traffic not restricted by source). If this field is present and contains at least one item, this rule allows traffic only if the traffic matches at least one item in the from list.",
        ),
    ] = None
    ports: Annotated[
        Optional[List[NetworkPolicyPort]],
        Field(
            description="ports is a list of ports which should be made accessible on the pods selected for this rule. Each item in this list is combined using a logical OR. If this field is empty or missing, this rule matches all ports (traffic not restricted by port). If this field is present and contains at least one item, then this rule allows traffic only if the traffic matches at least one port in the list."
        ),
    ] = None


class NetworkPolicySpec(BaseModel):
    egress: Annotated[
        Optional[List[NetworkPolicyEgressRule]],
        Field(
            description="egress is a list of egress rules to be applied to the selected pods. Outgoing traffic is allowed if there are no NetworkPolicies selecting the pod (and cluster policy otherwise allows the traffic), OR if the traffic matches at least one egress rule across all of the NetworkPolicy objects whose podSelector matches the pod. If this field is empty then this NetworkPolicy limits all outgoing traffic (and serves solely to ensure that the pods it selects are isolated by default). This field is beta-level in 1.8"
        ),
    ] = None
    ingress: Annotated[
        Optional[List[NetworkPolicyIngressRule]],
        Field(
            description="ingress is a list of ingress rules to be applied to the selected pods. Traffic is allowed to a pod if there are no NetworkPolicies selecting the pod (and cluster policy otherwise allows the traffic), OR if the traffic source is the pod's local node, OR if the traffic matches at least one ingress rule across all of the NetworkPolicy objects whose podSelector matches the pod. If this field is empty then this NetworkPolicy does not allow any traffic (and serves solely to ensure that the pods it selects are isolated by default)"
        ),
    ] = None
    pod_selector: Annotated[
        apimachinery.LabelSelector,
        Field(
            alias="podSelector",
            description="podSelector selects the pods to which this NetworkPolicy object applies. The array of ingress rules is applied to any pods selected by this field. Multiple network policies can select the same set of pods. In this case, the ingress rules for each are combined additively. This field is NOT optional and follows standard label selector semantics. An empty podSelector matches all pods in this namespace.",
        ),
    ]
    policy_types: Annotated[
        Optional[List[str]],
        Field(
            alias="policyTypes",
            description='policyTypes is a list of rule types that the NetworkPolicy relates to. Valid options are ["Ingress"], ["Egress"], or ["Ingress", "Egress"]. If this field is not specified, it will default based on the existence of ingress or egress rules; policies that contain an egress section are assumed to affect egress, and all policies (whether or not they contain an ingress section) are assumed to affect ingress. If you want to write an egress-only policy, you must explicitly specify policyTypes [ "Egress" ]. Likewise, if you want to write a policy that specifies that no egress is allowed, you must specify a policyTypes value that include "Egress" (since such a policy would not include an egress section and would otherwise default to just [ "Ingress" ]). This field is beta-level in 1.8',
        ),
    ] = None


class Ingress(Resource):
    api_version: Annotated[
        Optional[Literal["networking.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "networking.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["Ingress"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "Ingress"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[IngressSpec],
        Field(
            description="spec is the desired state of the Ingress. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None
    status: Annotated[
        Optional[IngressStatus],
        Field(
            description="status is the current state of the Ingress. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None


IngressList = ResourceList["Ingress"]


class NetworkPolicy(Resource):
    api_version: Annotated[
        Optional[Literal["networking.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "networking.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["NetworkPolicy"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "NetworkPolicy"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[NetworkPolicySpec],
        Field(
            description="spec represents the specification of the desired behavior for this NetworkPolicy."
        ),
    ] = None


NetworkPolicyList = ResourceList["NetworkPolicy"]



"""Tests for cloudcoil package."""

from importlib.metadata import version as get_version

import pytest

from cloudcoil.apimachinery import ObjectMeta
from cloudcoil.models.kubernetes.core import v1 as corev1

version = ".".join(get_version("cloudcoil.models.kubernetes").split(".")[:3])


@pytest.mark.configure_test_cluster(
    cluster_name=f"test-cloudcoil-v{version}", version=f"v{version}", remove=False
)
def test_e2e(test_config):
    with test_config:
        assert corev1.Service.get("kubernetes", "default").metadata.name == "kubernetes"
        output = corev1.Namespace(metadata=ObjectMeta(generate_name="test-")).create()
        name = output.metadata.name
        assert corev1.Namespace.get(name).metadata.name == name
        output.metadata.annotations = {"test": "test"}
        output = output.update()
        assert output.metadata.annotations == {"test": "test"}
        assert output.remove(dry_run=True).metadata.name == name
        assert corev1.Namespace.delete(name, grace_period_seconds=0).status.phase == "Terminating"
        assert len(corev1.Pod.list(all_namespaces=True, limit=1)) > 1
        assert len(corev1.Pod.list(all_namespaces=True, limit=1).items) == 1


@pytest.mark.configure_test_cluster(
    cluster_name=f"test-cloudcoil-v{version}", remove=False, version=f"v{version}"
)
async def test_async_e2e(test_config):
    with test_config:
        assert (
            await corev1.Service.async_get("kubernetes", "default")
        ).metadata.name == "kubernetes"
        output = await corev1.Namespace(metadata=ObjectMeta(generate_name="test-")).async_create()
        name = output.metadata.name
        assert (await corev1.Namespace.async_get(name)).metadata.name == name
        output.metadata.annotations = {"test": "test"}
        output = await output.async_update()
        assert output.metadata.annotations == {"test": "test"}
        assert (await output.async_remove(dry_run=True)).metadata.name == name
        assert (
            await corev1.Namespace.async_delete(name, grace_period_seconds=0)
        ).status.phase == "Terminating"
        assert len(await corev1.Pod.async_list(all_namespaces=True, limit=1)) > 1
        assert len((await corev1.Pod.async_list(all_namespaces=True, limit=1)).items) == 1



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, Dict, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource


class NonResourceAttributes(BaseModel):
    path: Annotated[Optional[str], Field(description="Path is the URL path of the request")] = None
    verb: Annotated[Optional[str], Field(description="Verb is the standard HTTP verb")] = None


class NonResourceRule(BaseModel):
    non_resource_ur_ls: Annotated[
        Optional[List[str]],
        Field(
            alias="nonResourceURLs",
            description='NonResourceURLs is a set of partial urls that a user should have access to.  *s are allowed, but only as the full, final step in the path.  "*" means all.',
        ),
    ] = None
    verbs: Annotated[
        List[str],
        Field(
            description='Verb is a list of kubernetes non-resource API verbs, like: get, post, put, delete, patch, head, options.  "*" means all.'
        ),
    ]


class ResourceRule(BaseModel):
    api_groups: Annotated[
        Optional[List[str]],
        Field(
            alias="apiGroups",
            description='APIGroups is the name of the APIGroup that contains the resources.  If multiple API groups are specified, any action requested against one of the enumerated resources in any API group will be allowed.  "*" means all.',
        ),
    ] = None
    resource_names: Annotated[
        Optional[List[str]],
        Field(
            alias="resourceNames",
            description='ResourceNames is an optional white list of names that the rule applies to.  An empty set means that everything is allowed.  "*" means all.',
        ),
    ] = None
    resources: Annotated[
        Optional[List[str]],
        Field(
            description='Resources is a list of resources this rule applies to.  "*" means all in the specified apiGroups.\n "*/foo" represents the subresource \'foo\' for all resources in the specified apiGroups.'
        ),
    ] = None
    verbs: Annotated[
        List[str],
        Field(
            description='Verb is a list of kubernetes resource API verbs, like: get, list, watch, create, update, delete, proxy.  "*" means all.'
        ),
    ]


class SelfSubjectRulesReviewSpec(BaseModel):
    namespace: Annotated[
        Optional[str], Field(description="Namespace to evaluate rules for. Required.")
    ] = None


class SubjectAccessReviewStatus(BaseModel):
    allowed: Annotated[
        bool,
        Field(
            description="Allowed is required. True if the action would be allowed, false otherwise."
        ),
    ]
    denied: Annotated[
        Optional[bool],
        Field(
            description="Denied is optional. True if the action would be denied, otherwise false. If both allowed is false and denied is false, then the authorizer has no opinion on whether to authorize the action. Denied may not be true if Allowed is true."
        ),
    ] = None
    evaluation_error: Annotated[
        Optional[str],
        Field(
            alias="evaluationError",
            description="EvaluationError is an indication that some error occurred during the authorization check. It is entirely possible to get an error and be able to continue determine authorization status in spite of it. For instance, RBAC can be missing a role, but enough roles are still present and bound to reason about the request.",
        ),
    ] = None
    reason: Annotated[
        Optional[str],
        Field(description="Reason is optional.  It indicates why a request was allowed or denied."),
    ] = None


class SubjectRulesReviewStatus(BaseModel):
    evaluation_error: Annotated[
        Optional[str],
        Field(
            alias="evaluationError",
            description="EvaluationError can appear in combination with Rules. It indicates an error occurred during rule evaluation, such as an authorizer that doesn't support rule evaluation, and that ResourceRules and/or NonResourceRules may be incomplete.",
        ),
    ] = None
    incomplete: Annotated[
        bool,
        Field(
            description="Incomplete is true when the rules returned by this call are incomplete. This is most commonly encountered when an authorizer, such as an external authorizer, doesn't support rules evaluation."
        ),
    ]
    non_resource_rules: Annotated[
        List[NonResourceRule],
        Field(
            alias="nonResourceRules",
            description="NonResourceRules is the list of actions the subject is allowed to perform on non-resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.",
        ),
    ]
    resource_rules: Annotated[
        List[ResourceRule],
        Field(
            alias="resourceRules",
            description="ResourceRules is the list of actions the subject is allowed to perform on resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.",
        ),
    ]


class FieldSelectorAttributes(BaseModel):
    raw_selector: Annotated[
        Optional[str],
        Field(
            alias="rawSelector",
            description="rawSelector is the serialization of a field selector that would be included in a query parameter. Webhook implementations are encouraged to ignore rawSelector. The kube-apiserver's *SubjectAccessReview will parse the rawSelector as long as the requirements are not present.",
        ),
    ] = None
    requirements: Annotated[
        Optional[List[apimachinery.FieldSelectorRequirement]],
        Field(
            description="requirements is the parsed interpretation of a field selector. All requirements must be met for a resource instance to match the selector. Webhook implementations should handle requirements, but how to handle them is up to the webhook. Since requirements can only limit the request, it is safe to authorize as unlimited request if the requirements are not understood."
        ),
    ] = None


class LabelSelectorAttributes(BaseModel):
    raw_selector: Annotated[
        Optional[str],
        Field(
            alias="rawSelector",
            description="rawSelector is the serialization of a field selector that would be included in a query parameter. Webhook implementations are encouraged to ignore rawSelector. The kube-apiserver's *SubjectAccessReview will parse the rawSelector as long as the requirements are not present.",
        ),
    ] = None
    requirements: Annotated[
        Optional[List[apimachinery.LabelSelectorRequirement]],
        Field(
            description="requirements is the parsed interpretation of a label selector. All requirements must be met for a resource instance to match the selector. Webhook implementations should handle requirements, but how to handle them is up to the webhook. Since requirements can only limit the request, it is safe to authorize as unlimited request if the requirements are not understood."
        ),
    ] = None


class ResourceAttributes(BaseModel):
    field_selector: Annotated[
        Optional[FieldSelectorAttributes],
        Field(
            alias="fieldSelector",
            description="fieldSelector describes the limitation on access based on field.  It can only limit access, not broaden it.\n\nThis field  is alpha-level. To use this field, you must enable the `AuthorizeWithSelectors` feature gate (disabled by default).",
        ),
    ] = None
    group: Annotated[
        Optional[str],
        Field(description='Group is the API Group of the Resource.  "*" means all.'),
    ] = None
    label_selector: Annotated[
        Optional[LabelSelectorAttributes],
        Field(
            alias="labelSelector",
            description="labelSelector describes the limitation on access based on labels.  It can only limit access, not broaden it.\n\nThis field  is alpha-level. To use this field, you must enable the `AuthorizeWithSelectors` feature gate (disabled by default).",
        ),
    ] = None
    name: Annotated[
        Optional[str],
        Field(
            description='Name is the name of the resource being requested for a "get" or deleted for a "delete". "" (empty) means all.'
        ),
    ] = None
    namespace: Annotated[
        Optional[str],
        Field(
            description='Namespace is the namespace of the action being requested.  Currently, there is no distinction between no namespace and all namespaces "" (empty) is defaulted for LocalSubjectAccessReviews "" (empty) is empty for cluster-scoped resources "" (empty) means "all" for namespace scoped resources from a SubjectAccessReview or SelfSubjectAccessReview'
        ),
    ] = None
    resource: Annotated[
        Optional[str],
        Field(description='Resource is one of the existing resource types.  "*" means all.'),
    ] = None
    subresource: Annotated[
        Optional[str],
        Field(description='Subresource is one of the existing resource types.  "" means none.'),
    ] = None
    verb: Annotated[
        Optional[str],
        Field(
            description='Verb is a kubernetes resource API verb, like: get, list, watch, create, update, delete, proxy.  "*" means all.'
        ),
    ] = None
    version: Annotated[
        Optional[str],
        Field(description='Version is the API Version of the Resource.  "*" means all.'),
    ] = None


class SelfSubjectAccessReviewSpec(BaseModel):
    non_resource_attributes: Annotated[
        Optional[NonResourceAttributes],
        Field(
            alias="nonResourceAttributes",
            description="NonResourceAttributes describes information for a non-resource access request",
        ),
    ] = None
    resource_attributes: Annotated[
        Optional[ResourceAttributes],
        Field(
            alias="resourceAttributes",
            description="ResourceAuthorizationAttributes describes information for a resource access request",
        ),
    ] = None


class SubjectAccessReviewSpec(BaseModel):
    extra: Annotated[
        Optional[Dict[str, List[str]]],
        Field(
            description="Extra corresponds to the user.Info.GetExtra() method from the authenticator.  Since that is input to the authorizer it needs a reflection here."
        ),
    ] = None
    groups: Annotated[
        Optional[List[str]],
        Field(description="Groups is the groups you're testing for."),
    ] = None
    non_resource_attributes: Annotated[
        Optional[NonResourceAttributes],
        Field(
            alias="nonResourceAttributes",
            description="NonResourceAttributes describes information for a non-resource access request",
        ),
    ] = None
    resource_attributes: Annotated[
        Optional[ResourceAttributes],
        Field(
            alias="resourceAttributes",
            description="ResourceAuthorizationAttributes describes information for a resource access request",
        ),
    ] = None
    uid: Annotated[
        Optional[str], Field(description="UID information about the requesting user.")
    ] = None
    user: Annotated[
        Optional[str],
        Field(
            description='User is the user you\'re testing for. If you specify "User" but not "Groups", then is it interpreted as "What if User were not a member of any groups'
        ),
    ] = None


class LocalSubjectAccessReview(Resource):
    api_version: Annotated[
        Optional[Literal["authorization.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "authorization.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["LocalSubjectAccessReview"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "LocalSubjectAccessReview"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        SubjectAccessReviewSpec,
        Field(
            description="Spec holds information about the request being evaluated.  spec.namespace must be equal to the namespace you made the request against.  If empty, it is defaulted."
        ),
    ]
    status: Annotated[
        Optional[SubjectAccessReviewStatus],
        Field(
            description="Status is filled in by the server and indicates whether the request is allowed or not"
        ),
    ] = None


class SelfSubjectAccessReview(Resource):
    api_version: Annotated[
        Optional[Literal["authorization.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "authorization.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["SelfSubjectAccessReview"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "SelfSubjectAccessReview"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        SelfSubjectAccessReviewSpec,
        Field(
            description="Spec holds information about the request being evaluated.  user and groups must be empty"
        ),
    ]
    status: Annotated[
        Optional[SubjectAccessReviewStatus],
        Field(
            description="Status is filled in by the server and indicates whether the request is allowed or not"
        ),
    ] = None


class SelfSubjectRulesReview(Resource):
    api_version: Annotated[
        Optional[Literal["authorization.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "authorization.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["SelfSubjectRulesReview"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "SelfSubjectRulesReview"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        SelfSubjectRulesReviewSpec,
        Field(description="Spec holds information about the request being evaluated."),
    ]
    status: Annotated[
        Optional[SubjectRulesReviewStatus],
        Field(
            description="Status is filled in by the server and indicates the set of actions a user can perform."
        ),
    ] = None


class SubjectAccessReview(Resource):
    api_version: Annotated[
        Optional[Literal["authorization.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "authorization.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["SubjectAccessReview"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "SubjectAccessReview"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        SubjectAccessReviewSpec,
        Field(description="Spec holds information about the request being evaluated"),
    ]
    status: Annotated[
        Optional[SubjectAccessReviewStatus],
        Field(
            description="Status is filled in by the server and indicates whether the request is allowed or not"
        ),
    ] = None



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList


class APIServiceCondition(BaseModel):
    last_transition_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastTransitionTime",
            description="Last time the condition transitioned from one status to another.",
        ),
    ] = None
    message: Annotated[
        Optional[str],
        Field(description="Human-readable message indicating details about last transition."),
    ] = None
    reason: Annotated[
        Optional[str],
        Field(
            description="Unique, one-word, CamelCase reason for the condition's last transition."
        ),
    ] = None
    status: Annotated[
        str,
        Field(description="Status is the status of the condition. Can be True, False, Unknown."),
    ]
    type: Annotated[str, Field(description="Type is the type of the condition.")]


class APIServiceStatus(BaseModel):
    conditions: Annotated[
        Optional[List[APIServiceCondition]],
        Field(description="Current service state of apiService."),
    ] = None


class ServiceReference(BaseModel):
    name: Annotated[Optional[str], Field(description="Name is the name of the service")] = None
    namespace: Annotated[
        Optional[str], Field(description="Namespace is the namespace of the service")
    ] = None
    port: Annotated[
        Optional[int],
        Field(
            description="If specified, the port on the service that hosting webhook. Default to 443 for backward compatibility. `port` should be a valid port number (1-65535, inclusive)."
        ),
    ] = None


class APIServiceSpec(BaseModel):
    ca_bundle: Annotated[
        Optional[str],
        Field(
            alias="caBundle",
            description="CABundle is a PEM encoded CA bundle which will be used to validate an API server's serving certificate. If unspecified, system trust roots on the apiserver are used.",
        ),
    ] = None
    group: Annotated[
        Optional[str],
        Field(description="Group is the API group name this server hosts"),
    ] = None
    group_priority_minimum: Annotated[
        int,
        Field(
            alias="groupPriorityMinimum",
            description="GroupPriorityMinimum is the priority this group should have at least. Higher priority means that the group is preferred by clients over lower priority ones. Note that other versions of this group might specify even higher GroupPriorityMinimum values such that the whole group gets a higher priority. The primary sort is based on GroupPriorityMinimum, ordered highest number to lowest (20 before 10). The secondary sort is based on the alphabetical comparison of the name of the object.  (v1.bar before v1.foo) We'd recommend something like: *.k8s.io (except extensions) at 18000 and PaaSes (OpenShift, Deis) are recommended to be in the 2000s",
        ),
    ]
    insecure_skip_tls_verify: Annotated[
        Optional[bool],
        Field(
            alias="insecureSkipTLSVerify",
            description="InsecureSkipTLSVerify disables TLS certificate verification when communicating with this server. This is strongly discouraged.  You should use the CABundle instead.",
        ),
    ] = None
    service: Annotated[
        Optional[ServiceReference],
        Field(
            description="Service is a reference to the service for this API server.  It must communicate on port 443. If the Service is nil, that means the handling for the API groupversion is handled locally on this server. The call will simply delegate to the normal handler chain to be fulfilled."
        ),
    ] = None
    version: Annotated[
        Optional[str],
        Field(description='Version is the API version this server hosts.  For example, "v1"'),
    ] = None
    version_priority: Annotated[
        int,
        Field(
            alias="versionPriority",
            description='VersionPriority controls the ordering of this API version inside of its group.  Must be greater than zero. The primary sort is based on VersionPriority, ordered highest to lowest (20 before 10). Since it\'s inside of a group, the number can be small, probably in the 10s. In case of equal version priorities, the version string will be used to compute the order inside a group. If the version string is "kube-like", it will sort above non "kube-like" version strings, which are ordered lexicographically. "Kube-like" versions start with a "v", then are followed by a number (the major version), then optionally the string "alpha" or "beta" and another number (the minor version). These are sorted first by GA > beta > alpha (where GA is a version with no suffix such as beta or alpha), and then by comparing major version, then minor version. An example sorted list of versions: v10, v2, v1, v11beta2, v10beta3, v3beta1, v12alpha1, v11alpha2, foo1, foo10.',
        ),
    ]


class APIService(Resource):
    api_version: Annotated[
        Optional[Literal["apiregistration.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "apiregistration.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["APIService"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "APIService"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[APIServiceSpec],
        Field(description="Spec contains information for locating and communicating with a server"),
    ] = None
    status: Annotated[
        Optional[APIServiceStatus],
        Field(description="Status contains derived information about an API server"),
    ] = None


APIServiceList = ResourceList["APIService"]



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList


class ClusterTrustBundleSpec(BaseModel):
    signer_name: Annotated[
        Optional[str],
        Field(
            alias="signerName",
            description="signerName indicates the associated signer, if any.\n\nIn order to create or update a ClusterTrustBundle that sets signerName, you must have the following cluster-scoped permission: group=certificates.k8s.io resource=signers resourceName=<the signer name> verb=attest.\n\nIf signerName is not empty, then the ClusterTrustBundle object must be named with the signer name as a prefix (translating slashes to colons). For example, for the signer name `example.com/foo`, valid ClusterTrustBundle object names include `example.com:foo:abc` and `example.com:foo:v1`.\n\nIf signerName is empty, then the ClusterTrustBundle object's name must not have such a prefix.\n\nList/watch requests for ClusterTrustBundles can filter on this field using a `spec.signerName=NAME` field selector.",
        ),
    ] = None
    trust_bundle: Annotated[
        str,
        Field(
            alias="trustBundle",
            description="trustBundle contains the individual X.509 trust anchors for this bundle, as PEM bundle of PEM-wrapped, DER-formatted X.509 certificates.\n\nThe data must consist only of PEM certificate blocks that parse as valid X.509 certificates.  Each certificate must include a basic constraints extension with the CA bit set.  The API server will reject objects that contain duplicate certificates, or that use PEM block headers.\n\nUsers of ClusterTrustBundles, including Kubelet, are free to reorder and deduplicate certificate blocks in this file according to their own logic, as well as to drop PEM block headers and inter-block data.",
        ),
    ]


class ClusterTrustBundle(Resource):
    api_version: Annotated[
        Optional[Literal["certificates.k8s.io/v1alpha1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "certificates.k8s.io/v1alpha1"
    kind: Annotated[
        Optional[Literal["ClusterTrustBundle"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "ClusterTrustBundle"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(description="metadata contains the object metadata."),
    ] = None
    spec: Annotated[
        ClusterTrustBundleSpec,
        Field(description="spec contains the signer (if any) and trust anchors."),
    ]


ClusterTrustBundleList = ResourceList["ClusterTrustBundle"]



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList


class ExemptPriorityLevelConfiguration(BaseModel):
    lendable_percent: Annotated[
        Optional[int],
        Field(
            alias="lendablePercent",
            description="`lendablePercent` prescribes the fraction of the level's NominalCL that can be borrowed by other priority levels.  This value of this field must be between 0 and 100, inclusive, and it defaults to 0. The number of seats that other levels can borrow from this level, known as this level's LendableConcurrencyLimit (LendableCL), is defined as follows.\n\nLendableCL(i) = round( NominalCL(i) * lendablePercent(i)/100.0 )",
        ),
    ] = None
    nominal_concurrency_shares: Annotated[
        Optional[int],
        Field(
            alias="nominalConcurrencyShares",
            description="`nominalConcurrencyShares` (NCS) contributes to the computation of the NominalConcurrencyLimit (NominalCL) of this level. This is the number of execution seats nominally reserved for this priority level. This DOES NOT limit the dispatching from this priority level but affects the other priority levels through the borrowing mechanism. The server's concurrency limit (ServerCL) is divided among all the priority levels in proportion to their NCS values:\n\nNominalCL(i)  = ceil( ServerCL * NCS(i) / sum_ncs ) sum_ncs = sum[priority level k] NCS(k)\n\nBigger numbers mean a larger nominal concurrency limit, at the expense of every other priority level. This field has a default value of zero.",
        ),
    ] = None


class FlowDistinguisherMethod(BaseModel):
    type: Annotated[
        str,
        Field(
            description='`type` is the type of flow distinguisher method The supported types are "ByUser" and "ByNamespace". Required.'
        ),
    ]


class GroupSubject(BaseModel):
    name: Annotated[
        str,
        Field(
            description='name is the user group that matches, or "*" to match all user groups. See https://github.com/kubernetes/apiserver/blob/master/pkg/authentication/user/user.go for some well-known group names. Required.'
        ),
    ]


class NonResourcePolicyRule(BaseModel):
    non_resource_ur_ls: Annotated[
        List[str],
        Field(
            alias="nonResourceURLs",
            description='`nonResourceURLs` is a set of url prefixes that a user should have access to and may not be empty. For example:\n  - "/healthz" is legal\n  - "/hea*" is illegal\n  - "/hea" is legal but matches nothing\n  - "/hea/*" also matches nothing\n  - "/healthz/*" matches all per-component health checks.\n"*" matches all non-resource urls. if it is present, it must be the only entry. Required.',
        ),
    ]
    verbs: Annotated[
        List[str],
        Field(
            description='`verbs` is a list of matching verbs and may not be empty. "*" matches all verbs. If it is present, it must be the only entry. Required.'
        ),
    ]


class PriorityLevelConfigurationReference(BaseModel):
    name: Annotated[
        str,
        Field(
            description="`name` is the name of the priority level configuration being referenced Required."
        ),
    ]


class QueuingConfiguration(BaseModel):
    hand_size: Annotated[
        Optional[int],
        Field(
            alias="handSize",
            description="`handSize` is a small positive number that configures the shuffle sharding of requests into queues.  When enqueuing a request at this priority level the request's flow identifier (a string pair) is hashed and the hash value is used to shuffle the list of queues and deal a hand of the size specified here.  The request is put into one of the shortest queues in that hand. `handSize` must be no larger than `queues`, and should be significantly smaller (so that a few heavy flows do not saturate most of the queues).  See the user-facing documentation for more extensive guidance on setting this field.  This field has a default value of 8.",
        ),
    ] = None
    queue_length_limit: Annotated[
        Optional[int],
        Field(
            alias="queueLengthLimit",
            description="`queueLengthLimit` is the maximum number of requests allowed to be waiting in a given queue of this priority level at a time; excess requests are rejected.  This value must be positive.  If not specified, it will be defaulted to 50.",
        ),
    ] = None
    queues: Annotated[
        Optional[int],
        Field(
            description="`queues` is the number of queues for this priority level. The queues exist independently at each apiserver. The value must be positive.  Setting it to 1 effectively precludes shufflesharding and thus makes the distinguisher method of associated flow schemas irrelevant.  This field has a default value of 64."
        ),
    ] = None


class ResourcePolicyRule(BaseModel):
    api_groups: Annotated[
        List[str],
        Field(
            alias="apiGroups",
            description='`apiGroups` is a list of matching API groups and may not be empty. "*" matches all API groups and, if present, must be the only entry. Required.',
        ),
    ]
    cluster_scope: Annotated[
        Optional[bool],
        Field(
            alias="clusterScope",
            description="`clusterScope` indicates whether to match requests that do not specify a namespace (which happens either because the resource is not namespaced or the request targets all namespaces). If this field is omitted or false then the `namespaces` field must contain a non-empty list.",
        ),
    ] = None
    namespaces: Annotated[
        Optional[List[str]],
        Field(
            description='`namespaces` is a list of target namespaces that restricts matches.  A request that specifies a target namespace matches only if either (a) this list contains that target namespace or (b) this list contains "*".  Note that "*" matches any specified namespace but does not match a request that _does not specify_ a namespace (see the `clusterScope` field for that). This list may be empty, but only if `clusterScope` is true.'
        ),
    ] = None
    resources: Annotated[
        List[str],
        Field(
            description='`resources` is a list of matching resources (i.e., lowercase and plural) with, if desired, subresource.  For example, [ "services", "nodes/status" ].  This list may not be empty. "*" matches all resources and, if present, must be the only entry. Required.'
        ),
    ]
    verbs: Annotated[
        List[str],
        Field(
            description='`verbs` is a list of matching verbs and may not be empty. "*" matches all verbs and, if present, must be the only entry. Required.'
        ),
    ]


class ServiceAccountSubject(BaseModel):
    name: Annotated[
        str,
        Field(
            description='`name` is the name of matching ServiceAccount objects, or "*" to match regardless of name. Required.'
        ),
    ]
    namespace: Annotated[
        str,
        Field(
            description="`namespace` is the namespace of matching ServiceAccount objects. Required."
        ),
    ]


class UserSubject(BaseModel):
    name: Annotated[
        str,
        Field(
            description='`name` is the username that matches, or "*" to match all usernames. Required.'
        ),
    ]


class FlowSchemaCondition(BaseModel):
    last_transition_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastTransitionTime",
            description="`lastTransitionTime` is the last time the condition transitioned from one status to another.",
        ),
    ] = None
    message: Annotated[
        Optional[str],
        Field(
            description="`message` is a human-readable message indicating details about last transition."
        ),
    ] = None
    reason: Annotated[
        Optional[str],
        Field(
            description="`reason` is a unique, one-word, CamelCase reason for the condition's last transition."
        ),
    ] = None
    status: Annotated[
        Optional[str],
        Field(
            description="`status` is the status of the condition. Can be True, False, Unknown. Required."
        ),
    ] = None
    type: Annotated[
        Optional[str],
        Field(description="`type` is the type of the condition. Required."),
    ] = None


class FlowSchemaStatus(BaseModel):
    conditions: Annotated[
        Optional[List[FlowSchemaCondition]],
        Field(description="`conditions` is a list of the current states of FlowSchema."),
    ] = None


class LimitResponse(BaseModel):
    queuing: Annotated[
        Optional[QueuingConfiguration],
        Field(
            description='`queuing` holds the configuration parameters for queuing. This field may be non-empty only if `type` is `"Queue"`.'
        ),
    ] = None
    type: Annotated[
        str,
        Field(
            description='`type` is "Queue" or "Reject". "Queue" means that requests that can not be executed upon arrival are held in a queue until they can be executed or a queuing limit is reached. "Reject" means that requests that can not be executed upon arrival are rejected. Required.'
        ),
    ]


class LimitedPriorityLevelConfiguration(BaseModel):
    borrowing_limit_percent: Annotated[
        Optional[int],
        Field(
            alias="borrowingLimitPercent",
            description="`borrowingLimitPercent`, if present, configures a limit on how many seats this priority level can borrow from other priority levels. The limit is known as this level's BorrowingConcurrencyLimit (BorrowingCL) and is a limit on the total number of seats that this level may borrow at any one time. This field holds the ratio of that limit to the level's nominal concurrency limit. When this field is non-nil, it must hold a non-negative integer and the limit is calculated as follows.\n\nBorrowingCL(i) = round( NominalCL(i) * borrowingLimitPercent(i)/100.0 )\n\nThe value of this field can be more than 100, implying that this priority level can borrow a number of seats that is greater than its own nominal concurrency limit (NominalCL). When this field is left `nil`, the limit is effectively infinite.",
        ),
    ] = None
    lendable_percent: Annotated[
        Optional[int],
        Field(
            alias="lendablePercent",
            description="`lendablePercent` prescribes the fraction of the level's NominalCL that can be borrowed by other priority levels. The value of this field must be between 0 and 100, inclusive, and it defaults to 0. The number of seats that other levels can borrow from this level, known as this level's LendableConcurrencyLimit (LendableCL), is defined as follows.\n\nLendableCL(i) = round( NominalCL(i) * lendablePercent(i)/100.0 )",
        ),
    ] = None
    limit_response: Annotated[
        Optional[LimitResponse],
        Field(
            alias="limitResponse",
            description="`limitResponse` indicates what to do with requests that can not be executed right now",
        ),
    ] = None
    nominal_concurrency_shares: Annotated[
        Optional[int],
        Field(
            alias="nominalConcurrencyShares",
            description='`nominalConcurrencyShares` (NCS) contributes to the computation of the NominalConcurrencyLimit (NominalCL) of this level. This is the number of execution seats available at this priority level. This is used both for requests dispatched from this priority level as well as requests dispatched from other priority levels borrowing seats from this level. The server\'s concurrency limit (ServerCL) is divided among the Limited priority levels in proportion to their NCS values:\n\nNominalCL(i)  = ceil( ServerCL * NCS(i) / sum_ncs ) sum_ncs = sum[priority level k] NCS(k)\n\nBigger numbers mean a larger nominal concurrency limit, at the expense of every other priority level.\n\nIf not specified, this field defaults to a value of 30.\n\nSetting this field to zero supports the construction of a "jail" for this priority level that is used to hold some request(s)',
        ),
    ] = None


class PriorityLevelConfigurationCondition(BaseModel):
    last_transition_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastTransitionTime",
            description="`lastTransitionTime` is the last time the condition transitioned from one status to another.",
        ),
    ] = None
    message: Annotated[
        Optional[str],
        Field(
            description="`message` is a human-readable message indicating details about last transition."
        ),
    ] = None
    reason: Annotated[
        Optional[str],
        Field(
            description="`reason` is a unique, one-word, CamelCase reason for the condition's last transition."
        ),
    ] = None
    status: Annotated[
        Optional[str],
        Field(
            description="`status` is the status of the condition. Can be True, False, Unknown. Required."
        ),
    ] = None
    type: Annotated[
        Optional[str],
        Field(description="`type` is the type of the condition. Required."),
    ] = None


class PriorityLevelConfigurationSpec(BaseModel):
    exempt: Annotated[
        Optional[ExemptPriorityLevelConfiguration],
        Field(
            description='`exempt` specifies how requests are handled for an exempt priority level. This field MUST be empty if `type` is `"Limited"`. This field MAY be non-empty if `type` is `"Exempt"`. If empty and `type` is `"Exempt"` then the default values for `ExemptPriorityLevelConfiguration` apply.'
        ),
    ] = None
    limited: Annotated[
        Optional[LimitedPriorityLevelConfiguration],
        Field(
            description='`limited` specifies how requests are handled for a Limited priority level. This field must be non-empty if and only if `type` is `"Limited"`.'
        ),
    ] = None
    type: Annotated[
        str,
        Field(
            description='`type` indicates whether this priority level is subject to limitation on request execution.  A value of `"Exempt"` means that requests of this priority level are not subject to a limit (and thus are never queued) and do not detract from the capacity made available to other priority levels.  A value of `"Limited"` means that (a) requests of this priority level _are_ subject to limits and (b) some of the server\'s limited capacity is made available exclusively to this priority level. Required.'
        ),
    ]


class PriorityLevelConfigurationStatus(BaseModel):
    conditions: Annotated[
        Optional[List[PriorityLevelConfigurationCondition]],
        Field(description='`conditions` is the current state of "request-priority".'),
    ] = None


class Subject(BaseModel):
    group: Annotated[
        Optional[GroupSubject],
        Field(description="`group` matches based on user group name."),
    ] = None
    kind: Annotated[
        str,
        Field(description="`kind` indicates which one of the other fields is non-empty. Required"),
    ]
    service_account: Annotated[
        Optional[ServiceAccountSubject],
        Field(
            alias="serviceAccount",
            description="`serviceAccount` matches ServiceAccounts.",
        ),
    ] = None
    user: Annotated[
        Optional[UserSubject], Field(description="`user` matches based on username.")
    ] = None


class PolicyRulesWithSubjects(BaseModel):
    non_resource_rules: Annotated[
        Optional[List[NonResourcePolicyRule]],
        Field(
            alias="nonResourceRules",
            description="`nonResourceRules` is a list of NonResourcePolicyRules that identify matching requests according to their verb and the target non-resource URL.",
        ),
    ] = None
    resource_rules: Annotated[
        Optional[List[ResourcePolicyRule]],
        Field(
            alias="resourceRules",
            description="`resourceRules` is a slice of ResourcePolicyRules that identify matching requests according to their verb and the target resource. At least one of `resourceRules` and `nonResourceRules` has to be non-empty.",
        ),
    ] = None
    subjects: Annotated[
        List[Subject],
        Field(
            description="subjects is the list of normal user, serviceaccount, or group that this rule cares about. There must be at least one member in this slice. A slice that includes both the system:authenticated and system:unauthenticated user groups matches every request. Required."
        ),
    ]


class PriorityLevelConfiguration(Resource):
    api_version: Annotated[
        Optional[Literal["flowcontrol.apiserver.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "flowcontrol.apiserver.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["PriorityLevelConfiguration"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "PriorityLevelConfiguration"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="`metadata` is the standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[PriorityLevelConfigurationSpec],
        Field(
            description='`spec` is the specification of the desired behavior of a "request-priority". More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status'
        ),
    ] = None
    status: Annotated[
        Optional[PriorityLevelConfigurationStatus],
        Field(
            description='`status` is the current status of a "request-priority". More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status'
        ),
    ] = None


PriorityLevelConfigurationList = ResourceList["PriorityLevelConfiguration"]


class FlowSchemaSpec(BaseModel):
    distinguisher_method: Annotated[
        Optional[FlowDistinguisherMethod],
        Field(
            alias="distinguisherMethod",
            description="`distinguisherMethod` defines how to compute the flow distinguisher for requests that match this schema. `nil` specifies that the distinguisher is disabled and thus will always be the empty string.",
        ),
    ] = None
    matching_precedence: Annotated[
        Optional[int],
        Field(
            alias="matchingPrecedence",
            description="`matchingPrecedence` is used to choose among the FlowSchemas that match a given request. The chosen FlowSchema is among those with the numerically lowest (which we take to be logically highest) MatchingPrecedence.  Each MatchingPrecedence value must be ranged in [1,10000]. Note that if the precedence is not specified, it will be set to 1000 as default.",
        ),
    ] = None
    priority_level_configuration: Annotated[
        PriorityLevelConfigurationReference,
        Field(
            alias="priorityLevelConfiguration",
            description="`priorityLevelConfiguration` should reference a PriorityLevelConfiguration in the cluster. If the reference cannot be resolved, the FlowSchema will be ignored and marked as invalid in its status. Required.",
        ),
    ]
    rules: Annotated[
        Optional[List[PolicyRulesWithSubjects]],
        Field(
            description="`rules` describes which requests will match this flow schema. This FlowSchema matches a request if and only if at least one member of rules matches the request. if it is an empty slice, there will be no requests matching the FlowSchema."
        ),
    ] = None


class FlowSchema(Resource):
    api_version: Annotated[
        Optional[Literal["flowcontrol.apiserver.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "flowcontrol.apiserver.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["FlowSchema"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "FlowSchema"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="`metadata` is the standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[FlowSchemaSpec],
        Field(
            description="`spec` is the specification of the desired behavior of a FlowSchema. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None
    status: Annotated[
        Optional[FlowSchemaStatus],
        Field(
            description="`status` is the current status of a FlowSchema. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None


FlowSchemaList = ResourceList["FlowSchema"]



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, Dict, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList

from ..core import v1


class EndpointConditions(BaseModel):
    ready: Annotated[
        Optional[bool],
        Field(
            description='ready indicates that this endpoint is prepared to receive traffic, according to whatever system is managing the endpoint. A nil value indicates an unknown state. In most cases consumers should interpret this unknown state as ready. For compatibility reasons, ready should never be "true" for terminating endpoints, except when the normal readiness behavior is being explicitly overridden, for example when the associated Service has set the publishNotReadyAddresses flag.'
        ),
    ] = None
    serving: Annotated[
        Optional[bool],
        Field(
            description="serving is identical to ready except that it is set regardless of the terminating state of endpoints. This condition should be set to true for a ready endpoint that is terminating. If nil, consumers should defer to the ready condition."
        ),
    ] = None
    terminating: Annotated[
        Optional[bool],
        Field(
            description="terminating indicates that this endpoint is terminating. A nil value indicates an unknown state. Consumers should interpret this unknown state to mean that the endpoint is not terminating."
        ),
    ] = None


class EndpointPort(BaseModel):
    app_protocol: Annotated[
        Optional[str],
        Field(
            alias="appProtocol",
            description="The application protocol for this port. This is used as a hint for implementations to offer richer behavior for protocols that they understand. This field follows standard Kubernetes label syntax. Valid values are either:\n\n* Un-prefixed protocol names - reserved for IANA standard service names (as per RFC-6335 and https://www.iana.org/assignments/service-names).\n\n* Kubernetes-defined prefixed names:\n  * 'kubernetes.io/h2c' - HTTP/2 prior knowledge over cleartext as described in https://www.rfc-editor.org/rfc/rfc9113.html#name-starting-http-2-with-prior-\n  * 'kubernetes.io/ws'  - WebSocket over cleartext as described in https://www.rfc-editor.org/rfc/rfc6455\n  * 'kubernetes.io/wss' - WebSocket over TLS as described in https://www.rfc-editor.org/rfc/rfc6455\n\n* Other protocols should use implementation-defined prefixed names such as mycompany.com/my-custom-protocol.",
        ),
    ] = None
    name: Annotated[
        Optional[str],
        Field(
            description="name represents the name of this port. All ports in an EndpointSlice must have a unique name. If the EndpointSlice is derived from a Kubernetes service, this corresponds to the Service.ports[].name. Name must either be an empty string or pass DNS_LABEL validation: * must be no more than 63 characters long. * must consist of lower case alphanumeric characters or '-'. * must start and end with an alphanumeric character. Default is empty string."
        ),
    ] = None
    port: Annotated[
        Optional[int],
        Field(
            description="port represents the port number of the endpoint. If this is not specified, ports are not restricted and must be interpreted in the context of the specific consumer."
        ),
    ] = None
    protocol: Annotated[
        Optional[str],
        Field(
            description="protocol represents the IP protocol for this port. Must be UDP, TCP, or SCTP. Default is TCP."
        ),
    ] = None


class ForZone(BaseModel):
    name: Annotated[str, Field(description="name represents the name of the zone.")]


class EndpointHints(BaseModel):
    for_zones: Annotated[
        Optional[List[ForZone]],
        Field(
            alias="forZones",
            description="forZones indicates the zone(s) this endpoint should be consumed by to enable topology aware routing.",
        ),
    ] = None


class Endpoint(BaseModel):
    addresses: Annotated[
        List[str],
        Field(
            description="addresses of this endpoint. The contents of this field are interpreted according to the corresponding EndpointSlice addressType field. Consumers must handle different types of addresses in the context of their own capabilities. This must contain at least one address but no more than 100. These are all assumed to be fungible and clients may choose to only use the first element. Refer to: https://issue.k8s.io/106267"
        ),
    ]
    conditions: Annotated[
        Optional[EndpointConditions],
        Field(
            description="conditions contains information about the current status of the endpoint."
        ),
    ] = None
    deprecated_topology: Annotated[
        Optional[Dict[str, str]],
        Field(
            alias="deprecatedTopology",
            description="deprecatedTopology contains topology information part of the v1beta1 API. This field is deprecated, and will be removed when the v1beta1 API is removed (no sooner than kubernetes v1.24).  While this field can hold values, it is not writable through the v1 API, and any attempts to write to it will be silently ignored. Topology information can be found in the zone and nodeName fields instead.",
        ),
    ] = None
    hints: Annotated[
        Optional[EndpointHints],
        Field(
            description="hints contains information associated with how an endpoint should be consumed."
        ),
    ] = None
    hostname: Annotated[
        Optional[str],
        Field(
            description="hostname of this endpoint. This field may be used by consumers of endpoints to distinguish endpoints from each other (e.g. in DNS names). Multiple endpoints which use the same hostname should be considered fungible (e.g. multiple A values in DNS). Must be lowercase and pass DNS Label (RFC 1123) validation."
        ),
    ] = None
    node_name: Annotated[
        Optional[str],
        Field(
            alias="nodeName",
            description="nodeName represents the name of the Node hosting this endpoint. This can be used to determine endpoints local to a Node.",
        ),
    ] = None
    target_ref: Annotated[
        Optional[v1.ObjectReference],
        Field(
            alias="targetRef",
            description="targetRef is a reference to a Kubernetes object that represents this endpoint.",
        ),
    ] = None
    zone: Annotated[
        Optional[str],
        Field(description="zone is the name of the Zone this endpoint exists in."),
    ] = None


class EndpointSlice(Resource):
    address_type: Annotated[
        str,
        Field(
            alias="addressType",
            description="addressType specifies the type of address carried by this EndpointSlice. All addresses in this slice must be the same type. This field is immutable after creation. The following address types are currently supported: * IPv4: Represents an IPv4 Address. * IPv6: Represents an IPv6 Address. * FQDN: Represents a Fully Qualified Domain Name.",
        ),
    ]
    api_version: Annotated[
        Optional[Literal["discovery.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "discovery.k8s.io/v1"
    endpoints: Annotated[
        List[Endpoint],
        Field(
            description="endpoints is a list of unique endpoints in this slice. Each slice may include a maximum of 1000 endpoints."
        ),
    ]
    kind: Annotated[
        Optional[Literal["EndpointSlice"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "EndpointSlice"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(description="Standard object's metadata."),
    ] = None
    ports: Annotated[
        Optional[List[EndpointPort]],
        Field(
            description='ports specifies the list of network ports exposed by each endpoint in this slice. Each port must have a unique name. When ports is empty, it indicates that there are no defined ports. When a port is defined with a nil port value, it indicates "all ports". Each slice may include a maximum of 100 ports.'
        ),
    ] = None


EndpointSliceList = ResourceList["EndpointSlice"]



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, Dict, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource


class BoundObjectReference(BaseModel):
    api_version: Annotated[
        Optional[str],
        Field(alias="apiVersion", description="API version of the referent."),
    ] = None
    kind: Annotated[
        Optional[str],
        Field(description="Kind of the referent. Valid kinds are 'Pod' and 'Secret'."),
    ] = None
    name: Annotated[Optional[str], Field(description="Name of the referent.")] = None
    uid: Annotated[Optional[str], Field(description="UID of the referent.")] = None


class TokenRequestSpec(BaseModel):
    audiences: Annotated[
        List[str],
        Field(
            description="Audiences are the intendend audiences of the token. A recipient of a token must identify themself with an identifier in the list of audiences of the token, and otherwise should reject the token. A token issued for multiple audiences may be used to authenticate against any of the audiences listed but implies a high degree of trust between the target audiences."
        ),
    ]
    bound_object_ref: Annotated[
        Optional[BoundObjectReference],
        Field(
            alias="boundObjectRef",
            description="BoundObjectRef is a reference to an object that the token will be bound to. The token will only be valid for as long as the bound object exists. NOTE: The API server's TokenReview endpoint will validate the BoundObjectRef, but other audiences may not. Keep ExpirationSeconds small if you want prompt revocation.",
        ),
    ] = None
    expiration_seconds: Annotated[
        Optional[int],
        Field(
            alias="expirationSeconds",
            description="ExpirationSeconds is the requested duration of validity of the request. The token issuer may return a token with a different validity duration so a client needs to check the 'expiration' field in a response.",
        ),
    ] = None


class TokenReviewSpec(BaseModel):
    audiences: Annotated[
        Optional[List[str]],
        Field(
            description="Audiences is a list of the identifiers that the resource server presented with the token identifies as. Audience-aware token authenticators will verify that the token was intended for at least one of the audiences in this list. If no audiences are provided, the audience will default to the audience of the Kubernetes apiserver."
        ),
    ] = None
    token: Annotated[Optional[str], Field(description="Token is the opaque bearer token.")] = None


class UserInfo(BaseModel):
    extra: Annotated[
        Optional[Dict[str, List[str]]],
        Field(description="Any additional information provided by the authenticator."),
    ] = None
    groups: Annotated[
        Optional[List[str]],
        Field(description="The names of groups this user is a part of."),
    ] = None
    uid: Annotated[
        Optional[str],
        Field(
            description="A unique value that identifies this user across time. If this user is deleted and another user by the same name is added, they will have different UIDs."
        ),
    ] = None
    username: Annotated[
        Optional[str],
        Field(description="The name that uniquely identifies this user among all active users."),
    ] = None


class SelfSubjectReviewStatus(BaseModel):
    user_info: Annotated[
        Optional[UserInfo],
        Field(
            alias="userInfo",
            description="User attributes of the user making this request.",
        ),
    ] = None


class TokenRequestStatus(BaseModel):
    expiration_timestamp: Annotated[
        apimachinery.Time,
        Field(
            alias="expirationTimestamp",
            description="ExpirationTimestamp is the time of expiration of the returned token.",
        ),
    ]
    token: Annotated[str, Field(description="Token is the opaque bearer token.")]


class TokenReviewStatus(BaseModel):
    audiences: Annotated[
        Optional[List[str]],
        Field(
            description='Audiences are audience identifiers chosen by the authenticator that are compatible with both the TokenReview and token. An identifier is any identifier in the intersection of the TokenReviewSpec audiences and the token\'s audiences. A client of the TokenReview API that sets the spec.audiences field should validate that a compatible audience identifier is returned in the status.audiences field to ensure that the TokenReview server is audience aware. If a TokenReview returns an empty status.audience field where status.authenticated is "true", the token is valid against the audience of the Kubernetes API server.'
        ),
    ] = None
    authenticated: Annotated[
        Optional[bool],
        Field(
            description="Authenticated indicates that the token was associated with a known user."
        ),
    ] = None
    error: Annotated[
        Optional[str],
        Field(description="Error indicates that the token couldn't be checked"),
    ] = None
    user: Annotated[
        Optional[UserInfo],
        Field(description="User is the UserInfo associated with the provided token."),
    ] = None


class SelfSubjectReview(Resource):
    api_version: Annotated[
        Optional[Literal["authentication.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "authentication.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["SelfSubjectReview"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "SelfSubjectReview"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    status: Annotated[
        Optional[SelfSubjectReviewStatus],
        Field(description="Status is filled in by the server with the user attributes."),
    ] = None


class TokenRequest(Resource):
    api_version: Annotated[
        Optional[Literal["authentication.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "authentication.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["TokenRequest"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "TokenRequest"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        TokenRequestSpec,
        Field(description="Spec holds information about the request being evaluated"),
    ]
    status: Annotated[
        Optional[TokenRequestStatus],
        Field(
            description="Status is filled in by the server and indicates whether the token can be authenticated."
        ),
    ] = None


class TokenReview(Resource):
    api_version: Annotated[
        Optional[Literal["authentication.k8s.io/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "authentication.k8s.io/v1"
    kind: Annotated[
        Optional[Literal["TokenReview"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "TokenReview"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        TokenReviewSpec,
        Field(description="Spec holds information about the request being evaluated"),
    ]
    status: Annotated[
        Optional[TokenReviewStatus],
        Field(
            description="Status is filled in by the server and indicates whether the request can be authenticated."
        ),
    ] = None



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList


class ServerStorageVersion(BaseModel):
    api_server_id: Annotated[
        Optional[str],
        Field(alias="apiServerID", description="The ID of the reporting API server."),
    ] = None
    decodable_versions: Annotated[
        Optional[List[str]],
        Field(
            alias="decodableVersions",
            description="The API server can decode objects encoded in these versions. The encodingVersion must be included in the decodableVersions.",
        ),
    ] = None
    encoding_version: Annotated[
        Optional[str],
        Field(
            alias="encodingVersion",
            description="The API server encodes the object to this version when persisting it in the backend (e.g., etcd).",
        ),
    ] = None
    served_versions: Annotated[
        Optional[List[str]],
        Field(
            alias="servedVersions",
            description="The API server can serve these versions. DecodableVersions must include all ServedVersions.",
        ),
    ] = None


class StorageVersionSpec(BaseModel):
    pass


class StorageVersionCondition(BaseModel):
    last_transition_time: Annotated[
        Optional[apimachinery.Time],
        Field(
            alias="lastTransitionTime",
            description="Last time the condition transitioned from one status to another.",
        ),
    ] = None
    message: Annotated[
        str,
        Field(description="A human readable message indicating details about the transition."),
    ]
    observed_generation: Annotated[
        Optional[int],
        Field(
            alias="observedGeneration",
            description="If set, this represents the .metadata.generation that the condition was set based upon.",
        ),
    ] = None
    reason: Annotated[str, Field(description="The reason for the condition's last transition.")]
    status: Annotated[
        str, Field(description="Status of the condition, one of True, False, Unknown.")
    ]
    type: Annotated[str, Field(description="Type of the condition.")]


class StorageVersionStatus(BaseModel):
    common_encoding_version: Annotated[
        Optional[str],
        Field(
            alias="commonEncodingVersion",
            description="If all API server instances agree on the same encoding storage version, then this field is set to that version. Otherwise this field is left empty. API servers should finish updating its storageVersionStatus entry before serving write operations, so that this field will be in sync with the reality.",
        ),
    ] = None
    conditions: Annotated[
        Optional[List[StorageVersionCondition]],
        Field(description="The latest available observations of the storageVersion's state."),
    ] = None
    storage_versions: Annotated[
        Optional[List[ServerStorageVersion]],
        Field(
            alias="storageVersions",
            description="The reported versions per API server instance.",
        ),
    ] = None


class StorageVersion(Resource):
    api_version: Annotated[
        Optional[Literal["internal.apiserver.k8s.io/v1alpha1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "internal.apiserver.k8s.io/v1alpha1"
    kind: Annotated[
        Optional[Literal["StorageVersion"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "StorageVersion"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(description="The name is <group>.<resource>."),
    ] = None
    spec: Annotated[
        StorageVersionSpec,
        Field(description="Spec is an empty spec. It is here to comply with Kubernetes API style."),
    ]
    status: Annotated[
        StorageVersionStatus,
        Field(
            description="API server instances report the version they can decode and the version they encode objects to when persisting objects in the backend."
        ),
    ]


StorageVersionList = ResourceList["StorageVersion"]



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, Dict, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList

from ..core import v1


class CELDeviceSelector(BaseModel):
    expression: Annotated[
        str,
        Field(
            description='Expression is a CEL expression which evaluates a single device. It must evaluate to true when the device under consideration satisfies the desired criteria, and false when it does not. Any other result is an error and causes allocation of devices to abort.\n\nThe expression\'s input is an object named "device", which carries the following properties:\n - driver (string): the name of the driver which defines this device.\n - attributes (map[string]object): the device\'s attributes, grouped by prefix\n   (e.g. device.attributes["dra.example.com"] evaluates to an object with all\n   of the attributes which were prefixed by "dra.example.com".\n - capacity (map[string]object): the device\'s capacities, grouped by prefix.\n\nExample: Consider a device with driver="dra.example.com", which exposes two attributes named "model" and "ext.example.com/family" and which exposes one capacity named "modules". This input to this expression would have the following fields:\n\n    device.driver\n    device.attributes["dra.example.com"].model\n    device.attributes["ext.example.com"].family\n    device.capacity["dra.example.com"].modules\n\nThe device.driver field can be used to check for a specific driver, either as a high-level precondition (i.e. you only want to consider devices from this driver) or as part of a multi-clause expression that is meant to consider devices from different drivers.\n\nThe value type of each attribute is defined by the device definition, and users who write these expressions must consult the documentation for their specific drivers. The value type of each capacity is Quantity.\n\nIf an unknown prefix is used as a lookup in either device.attributes or device.capacity, an empty map will be returned. Any reference to an unknown field will cause an evaluation error and allocation to abort.\n\nA robust expression should check for the existence of attributes before referencing them.\n\nFor ease of use, the cel.bind() function is enabled, and can be used to simplify expressions that access multiple attributes with the same domain. For example:\n\n    cel.bind(dra, device.attributes["dra.example.com"], dra.someBool && dra.anotherBool)'
        ),
    ]


class DeviceAttribute(BaseModel):
    bool: Annotated[Optional[bool], Field(description="BoolValue is a true/false value.")] = None
    int: Annotated[Optional[int], Field(description="IntValue is a number.")] = None
    string: Annotated[
        Optional[str],
        Field(description="StringValue is a string. Must not be longer than 64 characters."),
    ] = None
    version: Annotated[
        Optional[str],
        Field(
            description="VersionValue is a semantic version according to semver.org spec 2.0.0. Must not be longer than 64 characters."
        ),
    ] = None


class DeviceConstraint(BaseModel):
    match_attribute: Annotated[
        Optional[str],
        Field(
            alias="matchAttribute",
            description='MatchAttribute requires that all devices in question have this attribute and that its type and value are the same across those devices.\n\nFor example, if you specified "dra.example.com/numa" (a hypothetical example!), then only devices in the same NUMA node will be chosen. A device which does not have that attribute will not be chosen. All devices should use a value of the same type for this attribute because that is part of its specification, but if one device doesn\'t, then it also will not be chosen.\n\nMust include the domain qualifier.',
        ),
    ] = None
    requests: Annotated[
        Optional[List[str]],
        Field(
            description="Requests is a list of the one or more requests in this claim which must co-satisfy this constraint. If a request is fulfilled by multiple devices, then all of the devices must satisfy the constraint. If this is not specified, this constraint applies to all requests in this claim."
        ),
    ] = None


class DeviceRequestAllocationResult(BaseModel):
    device: Annotated[
        str,
        Field(
            description="Device references one device instance via its name in the driver's resource pool. It must be a DNS label."
        ),
    ]
    driver: Annotated[
        str,
        Field(
            description="Driver specifies the name of the DRA driver whose kubelet plugin should be invoked to process the allocation once the claim is needed on a node.\n\nMust be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver."
        ),
    ]
    pool: Annotated[
        str,
        Field(
            description="This name together with the driver name and the device name field identify which device was allocated (`<driver name>/<pool name>/<device name>`).\n\nMust not be longer than 253 characters and may contain one or more DNS sub-domains separated by slashes."
        ),
    ]
    request: Annotated[
        str,
        Field(
            description="Request is the name of the request in the claim which caused this device to be allocated. Multiple devices may have been allocated per request."
        ),
    ]


class DeviceSelector(BaseModel):
    cel: Annotated[
        Optional[CELDeviceSelector],
        Field(description="CEL contains a CEL expression for selecting a device."),
    ] = None


class PodSchedulingContextSpec(BaseModel):
    potential_nodes: Annotated[
        Optional[List[str]],
        Field(
            alias="potentialNodes",
            description="PotentialNodes lists nodes where the Pod might be able to run.\n\nThe size of this field is limited to 128. This is large enough for many clusters. Larger clusters may need more attempts to find a node that suits all pending resources. This may get increased in the future, but not reduced.",
        ),
    ] = None
    selected_node: Annotated[
        Optional[str],
        Field(
            alias="selectedNode",
            description='SelectedNode is the node for which allocation of ResourceClaims that are referenced by the Pod and that use "WaitForFirstConsumer" allocation is to be attempted.',
        ),
    ] = None


class ResourceClaimConsumerReference(BaseModel):
    api_group: Annotated[
        Optional[str],
        Field(
            alias="apiGroup",
            description="APIGroup is the group for the resource being referenced. It is empty for the core API. This matches the group in the APIVersion that is used when creating the resources.",
        ),
    ] = None
    name: Annotated[str, Field(description="Name is the name of resource being referenced.")]
    resource: Annotated[
        str,
        Field(description='Resource is the type of resource being referenced, for example "pods".'),
    ]
    uid: Annotated[
        str,
        Field(description="UID identifies exactly one incarnation of the resource."),
    ]


class ResourceClaimSchedulingStatus(BaseModel):
    name: Annotated[
        str,
        Field(description="Name matches the pod.spec.resourceClaims[*].Name field."),
    ]
    unsuitable_nodes: Annotated[
        Optional[List[str]],
        Field(
            alias="unsuitableNodes",
            description="UnsuitableNodes lists nodes that the ResourceClaim cannot be allocated for.\n\nThe size of this field is limited to 128, the same as for PodSchedulingSpec.PotentialNodes. This may get increased in the future, but not reduced.",
        ),
    ] = None


class ResourcePool(BaseModel):
    generation: Annotated[
        int,
        Field(
            description="Generation tracks the change in a pool over time. Whenever a driver changes something about one or more of the resources in a pool, it must change the generation in all ResourceSlices which are part of that pool. Consumers of ResourceSlices should only consider resources from the pool with the highest generation number. The generation may be reset by drivers, which should be fine for consumers, assuming that all ResourceSlices in a pool are updated to match or deleted.\n\nCombined with ResourceSliceCount, this mechanism enables consumers to detect pools which are comprised of multiple ResourceSlices and are in an incomplete state."
        ),
    ]
    name: Annotated[
        str,
        Field(
            description="Name is used to identify the pool. For node-local devices, this is often the node name, but this is not required.\n\nIt must not be longer than 253 characters and must consist of one or more DNS sub-domains separated by slashes. This field is immutable."
        ),
    ]
    resource_slice_count: Annotated[
        int,
        Field(
            alias="resourceSliceCount",
            description="ResourceSliceCount is the total number of ResourceSlices in the pool at this generation number. Must be greater than zero.\n\nConsumers can use this to check whether they have seen all ResourceSlices belonging to the same pool.",
        ),
    ]


class BasicDevice(BaseModel):
    attributes: Annotated[
        Optional[Dict[str, DeviceAttribute]],
        Field(
            description="Attributes defines the set of attributes for this device. The name of each attribute must be unique in that set.\n\nThe maximum number of attributes and capacities combined is 32."
        ),
    ] = None
    capacity: Annotated[
        Optional[Dict[str, apimachinery.Quantity]],
        Field(
            description="Capacity defines the set of capacities for this device. The name of each capacity must be unique in that set.\n\nThe maximum number of attributes and capacities combined is 32."
        ),
    ] = None


class Device(BaseModel):
    basic: Annotated[
        Optional[BasicDevice], Field(description="Basic defines one device instance.")
    ] = None
    name: Annotated[
        str,
        Field(
            description="Name is unique identifier among all devices managed by the driver in the pool. It must be a DNS label."
        ),
    ]


class DeviceRequest(BaseModel):
    admin_access: Annotated[
        Optional[bool],
        Field(
            alias="adminAccess",
            description="AdminAccess indicates that this is a claim for administrative access to the device(s). Claims with AdminAccess are expected to be used for monitoring or other management services for a device.  They ignore all ordinary claims to the device with respect to access modes and any resource allocations.",
        ),
    ] = None
    allocation_mode: Annotated[
        Optional[str],
        Field(
            alias="allocationMode",
            description="AllocationMode and its related fields define how devices are allocated to satisfy this request. Supported values are:\n\n- ExactCount: This request is for a specific number of devices.\n  This is the default. The exact number is provided in the\n  count field.\n\n- All: This request is for all of the matching devices in a pool.\n  Allocation will fail if some devices are already allocated,\n  unless adminAccess is requested.\n\nIf AlloctionMode is not specified, the default mode is ExactCount. If the mode is ExactCount and count is not specified, the default count is one. Any other requests must specify this field.\n\nMore modes may get added in the future. Clients must refuse to handle requests with unknown modes.",
        ),
    ] = None
    count: Annotated[
        Optional[int],
        Field(
            description='Count is used only when the count mode is "ExactCount". Must be greater than zero. If AllocationMode is ExactCount and this field is not specified, the default is one.'
        ),
    ] = None
    device_class_name: Annotated[
        str,
        Field(
            alias="deviceClassName",
            description="DeviceClassName references a specific DeviceClass, which can define additional configuration and selectors to be inherited by this request.\n\nA class is required. Which classes are available depends on the cluster.\n\nAdministrators may use this to restrict which devices may get requested by only installing classes with selectors for permitted devices. If users are free to request anything without restrictions, then administrators can create an empty DeviceClass for users to reference.",
        ),
    ]
    name: Annotated[
        str,
        Field(
            description="Name can be used to reference this request in a pod.spec.containers[].resources.claims entry and in a constraint of the claim.\n\nMust be a DNS label."
        ),
    ]
    selectors: Annotated[
        Optional[List[DeviceSelector]],
        Field(
            description="Selectors define criteria which must be satisfied by a specific device in order for that device to be considered for this request. All selectors must be satisfied for a device to be considered."
        ),
    ] = None


class OpaqueDeviceConfiguration(BaseModel):
    driver: Annotated[
        str,
        Field(
            description="Driver is used to determine which kubelet plugin needs to be passed these configuration parameters.\n\nAn admission policy provided by the driver developer could use this to decide whether it needs to validate them.\n\nMust be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver."
        ),
    ]
    parameters: Annotated[
        apimachinery.RawExtension,
        Field(
            description='Parameters can contain arbitrary data. It is the responsibility of the driver developer to handle validation and versioning. Typically this includes self-identification and a version ("kind" + "apiVersion" for Kubernetes types), with conversion between different versions.'
        ),
    ]


class PodSchedulingContextStatus(BaseModel):
    resource_claims: Annotated[
        Optional[List[ResourceClaimSchedulingStatus]],
        Field(
            alias="resourceClaims",
            description='ResourceClaims describes resource availability for each pod.spec.resourceClaim entry where the corresponding ResourceClaim uses "WaitForFirstConsumer" allocation mode.',
        ),
    ] = None


class ResourceSliceSpec(BaseModel):
    all_nodes: Annotated[
        Optional[bool],
        Field(
            alias="allNodes",
            description="AllNodes indicates that all nodes have access to the resources in the pool.\n\nExactly one of NodeName, NodeSelector and AllNodes must be set.",
        ),
    ] = None
    devices: Annotated[
        Optional[List[Device]],
        Field(
            description="Devices lists some or all of the devices in this pool.\n\nMust not have more than 128 entries."
        ),
    ] = None
    driver: Annotated[
        str,
        Field(
            description="Driver identifies the DRA driver providing the capacity information. A field selector can be used to list only ResourceSlice objects with a certain driver name.\n\nMust be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver. This field is immutable."
        ),
    ]
    node_name: Annotated[
        Optional[str],
        Field(
            alias="nodeName",
            description="NodeName identifies the node which provides the resources in this pool. A field selector can be used to list only ResourceSlice objects belonging to a certain node.\n\nThis field can be used to limit access from nodes to ResourceSlices with the same node name. It also indicates to autoscalers that adding new nodes of the same type as some old node might also make new resources available.\n\nExactly one of NodeName, NodeSelector and AllNodes must be set. This field is immutable.",
        ),
    ] = None
    node_selector: Annotated[
        Optional[v1.NodeSelector],
        Field(
            alias="nodeSelector",
            description="NodeSelector defines which nodes have access to the resources in the pool, when that pool is not limited to a single node.\n\nMust use exactly one term.\n\nExactly one of NodeName, NodeSelector and AllNodes must be set.",
        ),
    ] = None
    pool: Annotated[
        ResourcePool,
        Field(description="Pool describes the pool that this ResourceSlice belongs to."),
    ]


class DeviceAllocationConfiguration(BaseModel):
    opaque: Annotated[
        Optional[OpaqueDeviceConfiguration],
        Field(description="Opaque provides driver-specific configuration parameters."),
    ] = None
    requests: Annotated[
        Optional[List[str]],
        Field(
            description="Requests lists the names of requests where the configuration applies. If empty, its applies to all requests."
        ),
    ] = None
    source: Annotated[
        str,
        Field(
            description="Source records whether the configuration comes from a class and thus is not something that a normal user would have been able to set or from a claim."
        ),
    ]


class DeviceAllocationResult(BaseModel):
    config: Annotated[
        Optional[List[DeviceAllocationConfiguration]],
        Field(
            description="This field is a combination of all the claim and class configuration parameters. Drivers can distinguish between those based on a flag.\n\nThis includes configuration parameters for drivers which have no allocated devices in the result because it is up to the drivers which configuration parameters they support. They can silently ignore unknown configuration parameters."
        ),
    ] = None
    results: Annotated[
        Optional[List[DeviceRequestAllocationResult]],
        Field(description="Results lists all allocated devices."),
    ] = None


class DeviceClaimConfiguration(BaseModel):
    opaque: Annotated[
        Optional[OpaqueDeviceConfiguration],
        Field(description="Opaque provides driver-specific configuration parameters."),
    ] = None
    requests: Annotated[
        Optional[List[str]],
        Field(
            description="Requests lists the names of requests where the configuration applies. If empty, it applies to all requests."
        ),
    ] = None


class DeviceClassConfiguration(BaseModel):
    opaque: Annotated[
        Optional[OpaqueDeviceConfiguration],
        Field(description="Opaque provides driver-specific configuration parameters."),
    ] = None


class DeviceClassSpec(BaseModel):
    config: Annotated[
        Optional[List[DeviceClassConfiguration]],
        Field(
            description="Config defines configuration parameters that apply to each device that is claimed via this class. Some classses may potentially be satisfied by multiple drivers, so each instance of a vendor configuration applies to exactly one driver.\n\nThey are passed to the driver, but are not considered while allocating the claim."
        ),
    ] = None
    selectors: Annotated[
        Optional[List[DeviceSelector]],
        Field(
            description="Each selector must be satisfied by a device which is claimed via this class."
        ),
    ] = None
    suitable_nodes: Annotated[
        Optional[v1.NodeSelector],
        Field(
            alias="suitableNodes",
            description="Only nodes matching the selector will be considered by the scheduler when trying to find a Node that fits a Pod when that Pod uses a claim that has not been allocated yet *and* that claim gets allocated through a control plane controller. It is ignored when the claim does not use a control plane controller for allocation.\n\nSetting this field is optional. If unset, all Nodes are candidates.\n\nThis is an alpha field and requires enabling the DRAControlPlaneController feature gate.",
        ),
    ] = None


class PodSchedulingContext(Resource):
    api_version: Annotated[
        Optional[Literal["resource.k8s.io/v1alpha3"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "resource.k8s.io/v1alpha3"
    kind: Annotated[
        Optional[Literal["PodSchedulingContext"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "PodSchedulingContext"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta], Field(description="Standard object metadata")
    ] = None
    spec: Annotated[
        PodSchedulingContextSpec,
        Field(description="Spec describes where resources for the Pod are needed."),
    ]
    status: Annotated[
        Optional[PodSchedulingContextStatus],
        Field(description="Status describes where resources for the Pod can be allocated."),
    ] = None


PodSchedulingContextList = ResourceList["PodSchedulingContext"]


class ResourceSlice(Resource):
    api_version: Annotated[
        Optional[Literal["resource.k8s.io/v1alpha3"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "resource.k8s.io/v1alpha3"
    kind: Annotated[
        Optional[Literal["ResourceSlice"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "ResourceSlice"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta], Field(description="Standard object metadata")
    ] = None
    spec: Annotated[
        ResourceSliceSpec,
        Field(
            description="Contains the information published by the driver.\n\nChanging the spec automatically increments the metadata.generation number."
        ),
    ]


ResourceSliceList = ResourceList["ResourceSlice"]


class AllocationResult(BaseModel):
    controller: Annotated[
        Optional[str],
        Field(
            description="Controller is the name of the DRA driver which handled the allocation. That driver is also responsible for deallocating the claim. It is empty when the claim can be deallocated without involving a driver.\n\nA driver may allocate devices provided by other drivers, so this driver name here can be different from the driver names listed for the results.\n\nThis is an alpha field and requires enabling the DRAControlPlaneController feature gate."
        ),
    ] = None
    devices: Annotated[
        Optional[DeviceAllocationResult],
        Field(description="Devices is the result of allocating devices."),
    ] = None
    node_selector: Annotated[
        Optional[v1.NodeSelector],
        Field(
            alias="nodeSelector",
            description="NodeSelector defines where the allocated resources are available. If unset, they are available everywhere.",
        ),
    ] = None


class DeviceClaim(BaseModel):
    config: Annotated[
        Optional[List[DeviceClaimConfiguration]],
        Field(
            description="This field holds configuration for multiple potential drivers which could satisfy requests in this claim. It is ignored while allocating the claim."
        ),
    ] = None
    constraints: Annotated[
        Optional[List[DeviceConstraint]],
        Field(
            description="These constraints must be satisfied by the set of devices that get allocated for the claim."
        ),
    ] = None
    requests: Annotated[
        Optional[List[DeviceRequest]],
        Field(
            description="Requests represent individual requests for distinct devices which must all be satisfied. If empty, nothing needs to be allocated."
        ),
    ] = None


class DeviceClass(Resource):
    api_version: Annotated[
        Optional[Literal["resource.k8s.io/v1alpha3"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "resource.k8s.io/v1alpha3"
    kind: Annotated[
        Optional[Literal["DeviceClass"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "DeviceClass"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta], Field(description="Standard object metadata")
    ] = None
    spec: Annotated[
        DeviceClassSpec,
        Field(
            description="Spec defines what can be allocated and how to configure it.\n\nThis is mutable. Consumers have to be prepared for classes changing at any time, either because they get updated or replaced. Claim allocations are done once based on whatever was set in classes at the time of allocation.\n\nChanging the spec automatically increments the metadata.generation number."
        ),
    ]


DeviceClassList = ResourceList["DeviceClass"]


class ResourceClaimSpec(BaseModel):
    controller: Annotated[
        Optional[str],
        Field(
            description="Controller is the name of the DRA driver that is meant to handle allocation of this claim. If empty, allocation is handled by the scheduler while scheduling a pod.\n\nMust be a DNS subdomain and should end with a DNS domain owned by the vendor of the driver.\n\nThis is an alpha field and requires enabling the DRAControlPlaneController feature gate."
        ),
    ] = None
    devices: Annotated[
        Optional[DeviceClaim],
        Field(description="Devices defines how to request devices."),
    ] = None


class ResourceClaimStatus(BaseModel):
    allocation: Annotated[
        Optional[AllocationResult],
        Field(description="Allocation is set once the claim has been allocated successfully."),
    ] = None
    deallocation_requested: Annotated[
        Optional[bool],
        Field(
            alias="deallocationRequested",
            description="Indicates that a claim is to be deallocated. While this is set, no new consumers may be added to ReservedFor.\n\nThis is only used if the claim needs to be deallocated by a DRA driver. That driver then must deallocate this claim and reset the field together with clearing the Allocation field.\n\nThis is an alpha field and requires enabling the DRAControlPlaneController feature gate.",
        ),
    ] = None
    reserved_for: Annotated[
        Optional[List[ResourceClaimConsumerReference]],
        Field(
            alias="reservedFor",
            description="ReservedFor indicates which entities are currently allowed to use the claim. A Pod which references a ResourceClaim which is not reserved for that Pod will not be started. A claim that is in use or might be in use because it has been reserved must not get deallocated.\n\nIn a cluster with multiple scheduler instances, two pods might get scheduled concurrently by different schedulers. When they reference the same ResourceClaim which already has reached its maximum number of consumers, only one pod can be scheduled.\n\nBoth schedulers try to add their pod to the claim.status.reservedFor field, but only the update that reaches the API server first gets stored. The other one fails with an error and the scheduler which issued it knows that it must put the pod back into the queue, waiting for the ResourceClaim to become usable again.\n\nThere can be at most 32 such reservations. This may get increased in the future, but not reduced.",
        ),
    ] = None


class ResourceClaimTemplateSpec(BaseModel):
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="ObjectMeta may contain labels and annotations that will be copied into the PVC when creating it. No other fields are allowed and will be rejected during validation."
        ),
    ] = None
    spec: Annotated[
        ResourceClaimSpec,
        Field(
            description="Spec for the ResourceClaim. The entire content is copied unchanged into the ResourceClaim that gets created from this template. The same fields as in a ResourceClaim are also valid here."
        ),
    ]


class ResourceClaim(Resource):
    api_version: Annotated[
        Optional[Literal["resource.k8s.io/v1alpha3"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "resource.k8s.io/v1alpha3"
    kind: Annotated[
        Optional[Literal["ResourceClaim"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "ResourceClaim"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta], Field(description="Standard object metadata")
    ] = None
    spec: Annotated[
        ResourceClaimSpec,
        Field(
            description="Spec describes what is being requested and how to configure it. The spec is immutable."
        ),
    ]
    status: Annotated[
        Optional[ResourceClaimStatus],
        Field(
            description="Status describes whether the claim is ready to use and what has been allocated."
        ),
    ] = None


ResourceClaimList = ResourceList["ResourceClaim"]


class ResourceClaimTemplate(Resource):
    api_version: Annotated[
        Optional[Literal["resource.k8s.io/v1alpha3"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "resource.k8s.io/v1alpha3"
    kind: Annotated[
        Optional[Literal["ResourceClaimTemplate"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "ResourceClaimTemplate"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta], Field(description="Standard object metadata")
    ] = None
    spec: Annotated[
        ResourceClaimTemplateSpec,
        Field(
            description="Describes the ResourceClaim that is to be generated.\n\nThis field is immutable. A ResourceClaim will get created by the control plane for a Pod when needed and then not get updated anymore."
        ),
    ]


ResourceClaimTemplateList = ResourceList["ResourceClaimTemplate"]



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList


class LeaseCandidateSpec(BaseModel):
    binary_version: Annotated[
        Optional[str],
        Field(
            alias="binaryVersion",
            description='BinaryVersion is the binary version. It must be in a semver format without leading `v`. This field is required when strategy is "OldestEmulationVersion"',
        ),
    ] = None
    emulation_version: Annotated[
        Optional[str],
        Field(
            alias="emulationVersion",
            description='EmulationVersion is the emulation version. It must be in a semver format without leading `v`. EmulationVersion must be less than or equal to BinaryVersion. This field is required when strategy is "OldestEmulationVersion"',
        ),
    ] = None
    lease_name: Annotated[
        str,
        Field(
            alias="leaseName",
            description="LeaseName is the name of the lease for which this candidate is contending. This field is immutable.",
        ),
    ]
    ping_time: Annotated[
        Optional[apimachinery.MicroTime],
        Field(
            alias="pingTime",
            description="PingTime is the last time that the server has requested the LeaseCandidate to renew. It is only done during leader election to check if any LeaseCandidates have become ineligible. When PingTime is updated, the LeaseCandidate will respond by updating RenewTime.",
        ),
    ] = None
    preferred_strategies: Annotated[
        List[str],
        Field(
            alias="preferredStrategies",
            description="PreferredStrategies indicates the list of strategies for picking the leader for coordinated leader election. The list is ordered, and the first strategy supersedes all other strategies. The list is used by coordinated leader election to make a decision about the final election strategy. This follows as - If all clients have strategy X as the first element in this list, strategy X will be used. - If a candidate has strategy [X] and another candidate has strategy [Y, X], Y supersedes X and strategy Y\n  will be used.\n- If a candidate has strategy [X, Y] and another candidate has strategy [Y, X], this is a user error and leader\n  election will not operate the Lease until resolved.\n(Alpha) Using this field requires the CoordinatedLeaderElection feature gate to be enabled.",
        ),
    ]
    renew_time: Annotated[
        Optional[apimachinery.MicroTime],
        Field(
            alias="renewTime",
            description="RenewTime is the time that the LeaseCandidate was last updated. Any time a Lease needs to do leader election, the PingTime field is updated to signal to the LeaseCandidate that they should update the RenewTime. Old LeaseCandidate objects are also garbage collected if it has been hours since the last renew. The PingTime field is updated regularly to prevent garbage collection for still active LeaseCandidates.",
        ),
    ] = None


class LeaseCandidate(Resource):
    api_version: Annotated[
        Optional[Literal["coordination.k8s.io/v1alpha1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "coordination.k8s.io/v1alpha1"
    kind: Annotated[
        Optional[Literal["LeaseCandidate"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "LeaseCandidate"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[LeaseCandidateSpec],
        Field(
            description="spec contains the specification of the Lease. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status"
        ),
    ] = None


LeaseCandidateList = ResourceList["LeaseCandidate"]



# Generated by cloudcoil-model-codegen v0.0.17
# DO NOT EDIT

from __future__ import annotations

from typing import Annotated, Dict, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil._pydantic import BaseModel
from cloudcoil.resources import Resource, ResourceList


class Eviction(Resource):
    api_version: Annotated[
        Optional[Literal["policy/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "policy/v1"
    delete_options: Annotated[
        Optional[apimachinery.DeleteOptions],
        Field(alias="deleteOptions", description="DeleteOptions may be provided"),
    ] = None
    kind: Annotated[
        Optional[Literal["Eviction"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "Eviction"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(description="ObjectMeta describes the pod that is being evicted."),
    ] = None


class PodDisruptionBudgetSpec(BaseModel):
    max_unavailable: Annotated[
        Optional[apimachinery.IntOrString],
        Field(
            alias="maxUnavailable",
            description='An eviction is allowed if at most "maxUnavailable" pods selected by "selector" are unavailable after the eviction, i.e. even in absence of the evicted pod. For example, one can prevent all voluntary evictions by specifying 0. This is a mutually exclusive setting with "minAvailable".',
        ),
    ] = None
    min_available: Annotated[
        Optional[apimachinery.IntOrString],
        Field(
            alias="minAvailable",
            description='An eviction is allowed if at least "minAvailable" pods selected by "selector" will still be available after the eviction, i.e. even in the absence of the evicted pod.  So for example you can prevent all voluntary evictions by specifying "100%".',
        ),
    ] = None
    selector: Annotated[
        Optional[apimachinery.LabelSelector],
        Field(
            description="Label query over pods whose evictions are managed by the disruption budget. A null selector will match no pods, while an empty ({}) selector will select all pods within the namespace."
        ),
    ] = None
    unhealthy_pod_eviction_policy: Annotated[
        Optional[str],
        Field(
            alias="unhealthyPodEvictionPolicy",
            description='UnhealthyPodEvictionPolicy defines the criteria for when unhealthy pods should be considered for eviction. Current implementation considers healthy pods, as pods that have status.conditions item with type="Ready",status="True".\n\nValid policies are IfHealthyBudget and AlwaysAllow. If no policy is specified, the default behavior will be used, which corresponds to the IfHealthyBudget policy.\n\nIfHealthyBudget policy means that running pods (status.phase="Running"), but not yet healthy can be evicted only if the guarded application is not disrupted (status.currentHealthy is at least equal to status.desiredHealthy). Healthy pods will be subject to the PDB for eviction.\n\nAlwaysAllow policy means that all running pods (status.phase="Running"), but not yet healthy are considered disrupted and can be evicted regardless of whether the criteria in a PDB is met. This means perspective running pods of a disrupted application might not get a chance to become healthy. Healthy pods will be subject to the PDB for eviction.\n\nAdditional policies may be added in the future. Clients making eviction decisions should disallow eviction of unhealthy pods if they encounter an unrecognized policy in this field.\n\nThis field is beta-level. The eviction API uses this field when the feature gate PDBUnhealthyPodEvictionPolicy is enabled (enabled by default).',
        ),
    ] = None


class PodDisruptionBudgetStatus(BaseModel):
    conditions: Annotated[
        Optional[List[apimachinery.Condition]],
        Field(
            description="Conditions contain conditions for PDB. The disruption controller sets the DisruptionAllowed condition. The following are known values for the reason field (additional reasons could be added in the future): - SyncFailed: The controller encountered an error and wasn't able to compute\n              the number of allowed disruptions. Therefore no disruptions are\n              allowed and the status of the condition will be False.\n- InsufficientPods: The number of pods are either at or below the number\n                    required by the PodDisruptionBudget. No disruptions are\n                    allowed and the status of the condition will be False.\n- SufficientPods: There are more pods than required by the PodDisruptionBudget.\n                  The condition will be True, and the number of allowed\n                  disruptions are provided by the disruptionsAllowed property."
        ),
    ] = None
    current_healthy: Annotated[
        int, Field(alias="currentHealthy", description="current number of healthy pods")
    ]
    desired_healthy: Annotated[
        int,
        Field(alias="desiredHealthy", description="minimum desired number of healthy pods"),
    ]
    disrupted_pods: Annotated[
        Optional[Dict[str, apimachinery.Time]],
        Field(
            alias="disruptedPods",
            description="DisruptedPods contains information about pods whose eviction was processed by the API server eviction subresource handler but has not yet been observed by the PodDisruptionBudget controller. A pod will be in this map from the time when the API server processed the eviction request to the time when the pod is seen by PDB controller as having been marked for deletion (or after a timeout). The key in the map is the name of the pod and the value is the time when the API server processed the eviction request. If the deletion didn't occur and a pod is still there it will be removed from the list automatically by PodDisruptionBudget controller after some time. If everything goes smooth this map should be empty for the most of the time. Large number of entries in the map may indicate problems with pod deletions.",
        ),
    ] = None
    disruptions_allowed: Annotated[
        int,
        Field(
            alias="disruptionsAllowed",
            description="Number of pod disruptions that are currently allowed.",
        ),
    ]
    expected_pods: Annotated[
        int,
        Field(
            alias="expectedPods",
            description="total number of pods counted by this disruption budget",
        ),
    ]
    observed_generation: Annotated[
        Optional[int],
        Field(
            alias="observedGeneration",
            description="Most recent generation observed when updating this PDB status. DisruptionsAllowed and other status information is valid only if observedGeneration equals to PDB's object generation.",
        ),
    ] = None


class PodDisruptionBudget(Resource):
    api_version: Annotated[
        Optional[Literal["policy/v1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "policy/v1"
    kind: Annotated[
        Optional[Literal["PodDisruptionBudget"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "PodDisruptionBudget"
    metadata: Annotated[
        Optional[apimachinery.ObjectMeta],
        Field(
            description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata"
        ),
    ] = None
    spec: Annotated[
        Optional[PodDisruptionBudgetSpec],
        Field(description="Specification of the desired behavior of the PodDisruptionBudget."),
    ] = None
    status: Annotated[
        Optional[PodDisruptionBudgetStatus],
        Field(description="Most recently observed status of the PodDisruptionBudget."),
    ] = None


PodDisruptionBudgetList = ResourceList["PodDisruptionBudget"]
